/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#ifndef ARCHITECTUREINTERFACE_H_
#define ARCHITECTUREINTERFACE_H_

#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "tivstand.h"
#include "tivcheck.h"

class ArchitectureInterface
{
public:
	UINT64	MemorySize;
	UINT64	DiskSize;
	CHAR *	CommonDirectory;
	CHAR *	TempDirectory;
	CHAR *	FinalDirectory;
	UINT32  NumberOfMachines;

	UINT32	NumberOfFiles;
	UINT32	MessageSize;
	
	void Initialize( UINT64		MemorySize,
			 UINT64		DiskSize,
			 const CHAR *	CommonDirectory,
			 const CHAR *	TempDirectory,
			 const CHAR *	FinalDirectory,
			 UINT32		NumberOfMachines )
	{
	   struct rlimit lim;
	   getrlimit( RLIMIT_NOFILE, &lim );
	   
		this->MemorySize = MemorySize;
		this->DiskSize = DiskSize;
		this->NumberOfMachines = NumberOfMachines;
		
		malloc_check( this->CommonDirectory, CHAR *, 1 + sizeof( CHAR ) * strlen( CommonDirectory ) );
		strcpy( this->CommonDirectory, CommonDirectory );
		malloc_check( this->TempDirectory, CHAR *, 1 + sizeof( CHAR ) * strlen( TempDirectory ) );
		strcpy( this->TempDirectory, TempDirectory );
		malloc_check( this->FinalDirectory, CHAR *, 1 + sizeof( CHAR ) * strlen( FinalDirectory ) );
		strcpy( this->FinalDirectory, FinalDirectory );
		
		this->MessageSize = 1000000;
		this->NumberOfFiles = lim.rlim_cur - 50;
	}
	
	INT32 TAG_ANY;
	INT32 SOURCE_ANY;
	
   virtual void initializeComm( UINT32 &myID, UINT32 &numComputers ) = 0;
   virtual void finalizeComm( ) = 0;
   virtual void sendMessage ( void *mess, INT32  lenAct, UINT32  toAct,   UINT32  tagAct ) = 0;
   virtual void probeMessage(                            INT32   fromReq, INT32   tagReq,
                                          INT32 &lenAct, UINT32 &fromAct, UINT32 &tagAct ) = 0;
   virtual void recvMessage ( void *mess, INT32  lenReq, INT32   fromReq, INT32   tagReq,
                                          INT32 &lenAct, UINT32 &fromAct, UINT32 &tagAct ) = 0;
	
	void SetMessageSize( UINT32 MessageSize )
	{
		this->MessageSize = MessageSize;
	}
};

#endif /*ARCHITECTUREINTERFACE_H_*/

