/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#ifndef PROBLEMINTERFACE_H_
#define PROBLEMINTERFACE_H_

#include <stdlib.h>

#include "tivstand.h"
#include "PathInterface.h"

class ProblemInterface
{
public:
	UINT64  SearchSize;
   UINT32  StateSize;
   UINT32  KeySize;
	UINT32  NumberOfGenerators;
	UINT32  BranchingFactor;
	UINT32  NumberOfRoots;
	UINT64  HashEntries;
	UINT32  HashSize;
	
	enum INVERSE_SETTINGS
	{
		INVERSES_NONE,
		INVERSES_AVAILABLE,
		INVERSES_MANDATORY
	} Inverses;
	
	void Initialize( 	UINT64	SearchSize,
						   UINT32   StateSize,
						   UINT32	NumberOfGenerators,
						   UINT32	BranchingFactor )
	{
		this->SearchSize = SearchSize;
		this->StateSize = StateSize;
		this->NumberOfGenerators = NumberOfGenerators;
		this->BranchingFactor = BranchingFactor;
		
      this->KeySize = StateSize;
		this->NumberOfRoots = 1;
		this->HashEntries = 0;
		this->HashSize = 0;
		this->Inverses = INVERSES_NONE;
	}
	
	void setKeySize( UINT32 KeySize )
	{
	   this->KeySize = KeySize;
	}
	
	void setNumberOfRoots( UINT32 NumberOfRoots )
	{
		this->NumberOfRoots = NumberOfRoots;
		assert( this->NumberOfRoots > 0 );
	}
	
	void setHashInfo( 	UINT32	HashSize,
						UINT64	HashEntries = 0 )
	{
		if( !HashEntries ) this->HashEntries = this->SearchSize;
		else this->HashEntries = HashEntries;
		this->HashSize = HashSize;
		
		// Sanity Check
		if( this->getHash( NULL, NULL ) ) assert( this->HashSize > 0 );
		if( this->inverseHash( NULL, NULL ) ) assert( this->getHash( NULL, NULL ) );
		assert( this->HashEntries >= this->SearchSize );
	}
	
	void setInverses( INVERSE_SETTINGS Inverses )
	{
		this->Inverses = Inverses;
	}
	
	virtual void getRoots(			BYTE *roots ) = 0;
	
	virtual bool getHash(			BYTE *hash,
									const BYTE *state )
	{
		return false;
	}
	
	virtual bool inverseHash( 		BYTE *state, 
									const BYTE *hash )
	{
		return false;
	}
	
	virtual void applyTargetGen( 	const BYTE *state, 
									      const UINT32 genLabel, 
									      BYTE *child, 
									      BOOL *success,
									      const SEARCH_PATH path = NULL,
									      const UINT32 pathSize = 0 ) = 0;
	
   virtual void applyAllGens( const BYTE *state, 
                              BYTE *child, 
                              BOOL *success,
                              const UINT32 count,
                              const SEARCH_PATH path = NULL,
                              const UINT32 pathSize = 0 )
   {
      for( UINT32 x = 0; x < count; x++ )
         for( UINT32 y = 0; y < NumberOfGenerators; y++ )
            applyTargetGen(  state + x * StateSize,
                             y,
                             child + x * StateSize * NumberOfGenerators + y * StateSize,
                             success + x * sizeof( BOOL ) * NumberOfGenerators + y * sizeof( BOOL ),
                             ( path ? path + x * pathSize : NULL ),
                             pathSize );
   }
};

#endif /*PROBLEMINTERFACE_H_*/
