/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

// The MPI lock
sem_t mpiLock;

static UINT32 messageSize( MPI_Status *status )
{
   int count;
   MPI_Get_count( status,  MPI_UNSIGNED_CHAR, &count );
   return count;
}

// Sends a message to a machine, will inform if it fails and when it succeeds
static void sendMessage( void *message, UINT32 length, INT8 to, UINT8 tag )
{
   sem_wait( &mpiLock );
   if( MPI_Send( message,
                 length,
                 MPI_UNSIGNED_CHAR,
                 to,
                 tag,
                 MPI_COMM_WORLD )!=MPI_SUCCESS )
   {
      log_string( 0, ( "Failed to send message %d to %d\n", tag, to ) );
      assert( 0 ); // Error Sending
   }
   sem_post( &mpiLock );
}


MPI_Status receiveMessage( void *buffer, MPI_Status status )
{
   sem_wait( &mpiLock );
   if( MPI_Recv( buffer,
                 messageSize( &status ),
                 MPI_UNSIGNED_CHAR,
                 status.MPI_SOURCE,
                 status.MPI_TAG,
                 MPI_COMM_WORLD,
                 &status )!=MPI_SUCCESS )
   {
      log_string( 0, ( "Error receiving message %d from %d\n",
                  status.MPI_TAG, status.MPI_SOURCE ) );
      assert( 0 );  // Error Receiving
   }
   sem_post( &mpiLock );
   return status;
}
                        
