/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#ifndef TIVHELPER_H
#define TIVHELPER_H

typedef struct location_entry
{
   vector <BYTE> *data;
   vector <UINT32> *locs;
   UINT32 loc;
} LOCATION_ENTRY;

// Stuff for array mergequeue
inline int locEntryComp( const LOCATION_ENTRY *l, const LOCATION_ENTRY *r )
{
   return (*(l->locs))[l->loc] - (*(r->locs))[r->loc];
}

inline bool locEntryNext( LOCATION_ENTRY *val )
{
   while( ++(val->loc) < val->locs->size() && (*(val->data))[val->loc] );
   return val->loc != val->locs->size();
}

inline UINT32 sigHashSlot( BYTE *val )
{
   UINT32 result = 0;
   for( UINT32 count = 0; count < problem.StateSize / SIZE32; count++ )
      result^= ( ( UINT32 * ) val )[count];
   
   return result;
}

inline int sigHashComp( BYTE *left, BYTE *right )
{
   return memcmp( left, right, problem.StateSize );
}

inline UINT32 locVal( LOCATION_ENTRY *val, UINT32 check )
{
   return ( val ? (*(val->locs))[val->loc] : blocks[check].length );
}

// Stuff for file mergequeue
typedef struct file_entry
{
   CHAR fname[1000];
   BOOL coll;
   FILE *file;
   BYTE *data;
   UINT32 size;
} FILE_ENTRY;

static int compSigComp( const void *left, const void *right )
{
   return memcmp( left, right, COMP_SIZE );
}

inline bool fileNext( FILE_ENTRY *entry )
{
   if( fread( entry->data, 1, entry->size, entry->file ) != entry->size )
   {
      free( entry->data );
      fclose( entry->file );
      if( entry->coll ) unlink( entry->fname );
      return false;
   }   
   return true;
}

inline int fileCompSigComp( const FILE_ENTRY *left, const FILE_ENTRY *right )
{
   return memcmp( left->data, right->data, COMP_SIZE );
}

inline int treeComp( const void *left, const void *right )
{
   return pathComp(((BYTE *)left) + COMP_SIZE, ((BYTE *)right) + COMP_SIZE);
}

inline int fileTreeComp( const FILE_ENTRY *left, const FILE_ENTRY *right )
{
   return pathComp( left->data + COMP_SIZE, right->data + COMP_SIZE );
}

// Get a block of a certain type and set it to another type
static inline UINT8 getBlock( )
{
   UINT8 resultBlock;

   for( resultBlock = BLOCK_FREE; resultBlock == BLOCK_FREE; usleep( 1000 ) )
      for( resultBlock = 0; resultBlock < BLOCK_FREE; resultBlock++ )
         if( blocks[resultBlock].used == false ) break;
   
   blocks[resultBlock].used = true;
   return resultBlock;
}

void advanceFile( FILE_QUEUE &queue )
{
   if( !queue.file || ftell( queue.file ) > 0 )
   {
      CHAR filename[1000];
      if( queue.file )
      {
         fclose( queue.file );
         log_string( 1, ( "Writing %s block %d, size %d, pathLen %d\n",
                     queue.name,
                     queue.cur,
                     queue.cnts[queue.cur], 
                     queue.maxs[queue.cur] ) );
      }
      queue.maxs.push_back( 0 );
      queue.cnts.push_back( 0 );
      sprintf( filename, "%s%s.%d.%d", 
               FINAL_DIR, queue.name, myRank, ++queue.cur );
      fopen_check( queue.file, filename, "w" );
   }
}


#endif

