/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson				  *
 *									  *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or	  *
 *  (at your option) any later version.					  *
 *									  *
 *  This program is distributed in the hope that it will be useful,	  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of	  *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	  *
 *  GNU General Public License for more details.			  *
 *									  *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include "mpi.h"

#include "architecture.h"
#include "problem.h"
#include "search.h"

#include "tivfile.h"

int buildFiles( ArchitectureInterface *arch )
{
   CHAR *computer, filename[1000];
   FILE *fp;
   UINT32 comp;
   vector <CHAR *> computers;
   
   // Read in the list of computers
   fopen_check( fp, "computers.txt", "r" );
   while( strcmp( computer = fread_word( fp ), "end" ) )
   {
      computers.push_back( NULL );
      malloc_check( computers[computers.size()-1], CHAR *, 20 );
      strcpy( computers[computers.size()-1], computer );
   }
   fclose( fp );
   
   printf( "There are %d known slaves running the computation\n",
      computers.size() );
   
   // Make the cleanup file, to get rid of results on the machines
   fopen_check( fp, "cleanup.sh", "w" );
   fprintf( fp, "for hostname in localhost" );
   for( comp = 0; comp < computers.size(); comp++ )
      fprintf( fp, " %s", computers[comp] );
   fprintf( fp, "\n" );
   fprintf( fp, "do\n" );
   fprintf( fp, "echo Working on $hostname\n" );
   fprintf( fp, "%s $hostname rm -rf %s %s\n", "ssh", arch->TempDirectory, arch->FinalDirectory );
   fprintf( fp, "done" );
   fclose( fp );
   system ( "chmod 777 cleanup.sh" );
   
   // Make the kill file, to kill ./a.out on all the processors
   fopen_check( fp, "kill.sh", "w" );
   fprintf( fp, "for hostname in localhost" );
   for( comp = 0; comp < computers.size(); comp++ )
      fprintf( fp, " %s", computers[comp] );
   fprintf( fp, "\n" );
   fprintf( fp, "do\n" );
   fprintf( fp, "echo Working on $hostname\n" );
   fprintf( fp, "%s $hostname pkill -9 enumerator 2>&1\n", "ssh" );
   fprintf( fp, "done" );
   fclose( fp );
   system ( "chmod 777 kill.sh" );
   
   // Make the folders file, to create folders on the remote machines
   fopen_check( fp, "folders.sh", "w" );
   fprintf( fp, "for hostname in localhost" );
   for( comp = 0; comp < computers.size(); comp++ )
      fprintf( fp, " %s", computers[comp] );
   fprintf( fp, "\n" );
   fprintf( fp, "do\n" );
   fprintf( fp, "echo Working on host $hostname\n" );
   fprintf( fp, "%s $hostname mkdir -p %s %s\n", "ssh", arch->FinalDirectory, arch->TempDirectory );
   fprintf( fp, "done" );
   fclose( fp );
   system ( "chmod 777 folders.sh" );
   
   // Make the procgroup file, to tell a.out which programs to run
   fopen_check( fp, "procgroup", "w" );
   fprintf( fp, "local\t0\n" );
   for( comp = 0; comp < computers.size(); comp++ )
      fprintf( fp, "%s\t1\t%senumerator 2>&1\n", computers[comp], arch->CommonDirectory );
   fclose( fp );
   
   return 0;
}

int main( int argc, char **argv )
{
   printf("enumerator : Copyright (C) 2007-2008  Eric Robinson\n"
    "This program comes with ABSOLUTELY NO WARRANTY.\n"
    "This is free software, and you are welcome to redistribute it\n"
    "under certain conditions; See the COPYING file of the distribution.\n\n");

   ArchitectureInterface *arch = new LocalhostMPI( argc, argv );
   ProblemInterface *prob = new VectorMatrixMultiplyGreasing( "Janko 4", arch );
   if( argc==2 && !strcmp( argv[1], "files" ) )
   {
      buildFiles(arch);
      return 0;
   }
   if( argc==2 && !strcmp( argv[1], "input" ) )
   {
      ( (VectorMatrixMultiplyGreasing * ) prob)
			->convertMatrices(arch->CommonDirectory);
      return 0;
   }
        ( (VectorMatrixMultiplyGreasing * ) prob)
			->loadMatrices(arch->CommonDirectory);
	SearchInterface *search = new TieredDDD( );
	( (GeneralTieredDDD * ) search)->SetDebugLevel( 1 );
	search->Run( prob, arch, search->RUN_ENUMERATE, &argc, &argv );
	return 0;
}
