/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#ifndef TIVMAIN_H
#define TIVMAIN_H

// Standard include files
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

// Configuration for our group
#include "tivstand.h"
#include "tivmath.h"

// Header include files
#include "tivcheck.h"
#include "tivlog.h"
#include "tivfile.h"
#include "tivvect.h"
#include "tivhash.h"
#include "tivbits.h"
#include "pqueue.h"

#include "PathInterface.h"
#include "ArchitectureInterface.h"
#include "config.h"
#include "architecture.h"
#include "problem.h"

ProblemInterface *problem = new VectorMatrixMultiplyGreasing( "janko" );

#include "files.h"

typedef struct blockAssign
{
   UINT32 proc;
   UINT32 store;
   UINT32 cur;
   UINT32 head;
   UINT32 tail;
} BLOCK_ASSIGN;

typedef struct hash
{
   BITVECTOR  hash;
   UINT64     size;
} HASH;

#define HASH_CHECK( loc, block, size )                                         \
   ( !HASH_MULTIPLE ||                                                         \
     BV_IS_SET( hash.hash,                                                     \
                getHash( &( block[ loc * size + SIZE32 ] ) ) % hash.size ) )
#define HASH_SET( loc, block, size )                                           \
   if( HASH_MULTIPLE )                                                         \
       BV_SET( hash.hash,                                                      \
               getHash( &( block[ loc * size + SIZE32 ] ) ) % hash.size )

/*
typedef struct pathNode
{
   BYTE sig[SIG_SIZE];
   STEP step;
} PATH_NODE;
*/

typedef struct fileQueue
{
   INT32 first;
   INT32 cur;
   vector <UINT32> maxs;
   vector <UINT32> cnts;
   FILE *file;
   CHAR *name;
} FILE_QUEUE;

enum STATE
{
   STATE_INIT,
   STATE_BFS,
   STATE_DDD,
   STATE_FINALIZE
};

// Global Data
STATE state = STATE_INIT;
BYTE *treeRoot;
BLOCK blocks[BLOCK_FREE];
BLOCK_ASSIGN ass = { BLOCK_FREE, BLOCK_FREE, BLOCK_FREE, 0, 0 };
FILE_QUEUE fins, cols;
HASH hash;

#ifndef HASH_FNS
   UINT8 getProcessor( BYTE *compSig )
   {
      UINT8 result = 0;
      for( UINT32 count = 0; count < COMP_SIZE; count++ ) result ^= compSig[count];
      return result;
   }
   
   UINT64 getHash( BYTE *compSig )
   {
      UINT64 result = 0;
      for( UINT32 count = 0; count < COMP_SIZE / SIZE64; count++ )
         result ^= ( ( UINT64 * )compSig )[count];
      return result;
   }
#endif

BYTE *getSig( UINT8 block, UINT32 number )
{
   return blocks[block].sigs + number * problem.StateSize;
}

PATH getPath( UINT8 block, UINT32 number )
{
   return blocks[block].paths + number * blocks[block].pathSize;
}

#endif

#include "comm.h"
#include "helper.h"
