/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#if MAX_BRANCH <= 256
   // Determine the appropriate upper bound to use
   #if MAX_BRANCH     <= 2
      #define STEP_MAX   2
      BYTE shifts[] = { 7, 6, 5, 4, 3, 2, 1, 0 };
   #elif MAX_BRANCH   <= 4
      #define STEP_MAX   4
      BYTE shifts[] = { 6, 4, 2, 0 };
   #elif MAX_BRANCH   <= 16
      #define STEP_MAX   16
      BYTE shifts[] = { 4, 0 };
   #else //MAX_BRANCH <= 256
      #define STEP_MAX   256
      BYTE shifts[] = { 0 };
   #endif
   
   // Set the appropriate data types and sizes
   #define PATH_BYTE     8
   #define PATH_WORD     4
   typedef BYTE *        PATH;
   typedef BYTE          STEP;
   UINT8 STEP_MASK =          STEP_MAX - 1;
   UINT8 STEPS_PER_BLOCK =    PATH_BYTE / log( 2, STEP_MAX );
   
   // Get a path value
   STEP GET_STEP( PATH path, UINT32 step )
   {
      assert( step < *( ( UINT32 * ) path ) );
      PATH addr = path + step / STEPS_PER_BLOCK + sizeof( UINT32 );
      BYTE shift = shifts[step % STEPS_PER_BLOCK ];
      return ( *addr & ( STEP_MASK << shift ) ) >> shift;
   }
   
   void SET_STEP( PATH path, UINT32 step, STEP val )
   {
      assert( step < *( ( UINT32 * ) path ) );
      assert( val < STEP_MAX );
      PATH addr = path + step / STEPS_PER_BLOCK + sizeof( UINT32 );
      BYTE shift = shifts[step % STEPS_PER_BLOCK ];
      *addr = ( *addr & ~( STEP_MASK << shift ) ) | ( val << shift );
   }

   UINT32 PATH_SIZE( UINT32 maxSteps )
   {
      if( maxSteps % ( STEPS_PER_BLOCK * PATH_WORD ) )
         return ( maxSteps / ( STEPS_PER_BLOCK * PATH_WORD ) + 2 ) * PATH_WORD;
      else return maxSteps / STEPS_PER_BLOCK + PATH_WORD;
   }
   
   UINT32 GET_STEPS( PATH path )
   {
      return *( ( UINT32 * ) path );
   }
   
   void SET_STEPS( PATH path, UINT32 steps )
   {
      *( ( UINT32 * ) path ) = steps;
   }
   
   INT32 pathComp( PATH left, PATH right )
   {
      UINT32 minSteps = MIN( GET_STEPS( left ), GET_STEPS( right ) ),
             maxBlock = minSteps / STEPS_PER_BLOCK;
      
      // Check up to the full block size for the shortest
      INT32 val = memcmp( left + SIZE32, right + SIZE32, maxBlock );
      if( val ) return val;
      
      // Check the remaining bytes
      for( UINT32 count = 0; count < minSteps % STEPS_PER_BLOCK; count++ )
      {
         val = GET_STEP( left, minSteps - minSteps % STEPS_PER_BLOCK + count ) - 
               GET_STEP( right, minSteps - minSteps % STEPS_PER_BLOCK + count );
         if( val ) return val;
      }
      
      // Find the longest and set it greater than the other
      if( GET_STEPS( left ) > GET_STEPS( right ) ) return 1;
      else if( GET_STEPS( right ) > GET_STEPS( left ) ) return -1;
      else return 0;
   }
      
#endif

