/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include <stdlib.h>

template <class T> class mergeQueue
{
public:
   mergeQueue( int (*comp)(const T *, const T *), 
               bool (*advance)(T *) )
   {
      this->comp = comp;
      this->advance = advance;
      this->nextTarget = NULL;
      this->data.push_back( NULL );
   }
   
   // Basic Constructor
   mergeQueue( int (*comp)(const T *, const T *), 
               bool (*advance)(T *),
               T *input,
               UINT32 size )
   {
      this->comp = comp;
      this->advance = advance;
      this->nextTarget = NULL;
      this->data.push_back( NULL );
      for( UINT32 count = 0; count < size; count++ )
         if( !this->advance || this->advance( input + count ) )
            this->push_local( input + count );
   }
   
   // Next Method
   inline T *pop( )
   {
      if( this->nextTarget && this->advance && this->advance( nextTarget ) )
         this->push_local( nextTarget );
      
      nextTarget = this->pop_local( );
      return nextTarget;
   }
   
private:
   inline T *pop_local( )
   {
      if( this->data.size() == 1 ) return NULL;
      if( !this->comp ) return this->data.pop_back();

      // Remove the first value and bump the last value up
      T *retval = this->data[1],
        *val = this->data.pop_back();
      if( this->data.size() > 1 ) this->data[1] = val;
       
      // Bubble it down
      for( UINT32 current = 1, left = current * 2, right = current * 2 + 1;
           left < this->data.size();
           left = current * 2, right = current * 2 + 1 )
      {
         if( right < this->data.size() && 
             (*(this->comp))( this->data[right], this->data[left] ) <= 0 )
         {
            if( (*(this->comp))( this->data[right], this->data[current] ) < 0 )
            {
               swap( this->data[current], this->data[right] );
               current = right;
            }
            else break;
         }
         else
         {
            if( (*(this->comp))( this->data[left], this->data[current] ) < 0 )
            {
               swap( this->data[current], this->data[left] );
               current = left;
            }
            else break;
         }
      }
      
      return retval;
   }
      
   inline void push_local( T *val )
   {
      // Add the value to the end of the queue
      this->data.push_back( val );
      
      if( this->comp )
      {
         // Bubble it up
         for( UINT32 current = this->data.size() - 1, parent = current / 2;
              parent;
              current = parent, parent = current / 2 )
         {
            if( (*(this->comp))( this->data[current], this->data[parent] ) < 0 )
            {
               swap( this->data[current], this->data[parent] );
            }
            else break;
         }
      }
   }

   T *nextTarget;
   int (*comp)(const T *, const T *);
   bool (*advance)(T *);
   vector <T *> data;
};

