
#define INPUT_DIR     "input/"
#define CONFIG_MAX    ( 1 << BAND_WIDTH )      // Number of configs per band
#define BAND_MAX      SIG_SIZE                 // The maximum number of bands

static BYTE *bands[2];                         // The matrices after loading


static void multiply( BYTE *target, BYTE *destination, unsigned char id )
{
   UINT32 block;
   UINT64 *source, *max, *dest;

   memset( destination, '\0', SIG_SIZE );
   
   for( block = 0; block < BAND_MAX; block++ )
   {
      // get the right spot
      source = ( UINT64 * )( bands[id] + block * CONFIG_MAX * SIG_SIZE + SIG_SIZE * target[block] );
      max = source + SIG_SIZE / CHUNK_BYTES;
      dest = ( UINT64 * )destination;
      // Add in everything from that location
      while( source < max ) *(dest++) ^= *(source++);
   }
}

// Convert the matrix from the previous notation to a binary
// format that we can read in quickly and doesn't consume
// a lot of disk space
   
static void sum_band( unsigned long num, unsigned long band, unsigned long config, BYTE *row_start )
{
 BYTE *destination = bands[num] + band * CONFIG_MAX * SIG_SIZE + config * SIG_SIZE;
  
 // Compute over all sets of columns
  for( unsigned long pos = 0; pos < SIG_SIZE; pos++ )
  {
      // Sum all the rows
      for( unsigned long row = 0; row < BAND_WIDTH; row++ )
     {
         if( ( config & ( 1 << ( BAND_WIDTH - row - 1 ) ) ) )
            *( destination + pos ) ^= *( row_start + SIG_SIZE * row + pos );
    }
   }
}

UINT32 getNextChild( BYTE *parent, 
                     BYTE *child, 
                     PATH  path, 
                     INT32 target )
{
   if( target==MAX_BRANCH ) target = 0;
   else target = target + 1;
   
   avoid( target, path );
   
   if( target==MAX_BRANCH )return MAX_BRANCH;
   
   if( parent!=NULL ) multiply( parent, child, target );
   return target;
}

UINT32 getTargetChild( BYTE *parent, BYTE *child, UINT32 target )
{
   if( target >= MAX_BRANCH ) return MAX_BRANCH;
   multiply( parent, child, target );
   return target;
}

void loadFirst( CHAR *filename, BYTE * location )
{
   FILE *file;
   INT8 input;
   
   fopen_check( file, filename, "r" );
   memset( location, '\0', SIG_SIZE );
   assert( getNext( file )=='[' );
   for( UINT32 count = 0; count < DIM; count++ )
   {
      if( ( input = getNext( file ) )=='1' ) BV_SET( location, count );
      else if( input!='0' ) assert( 0 );
   }
   if( getNext( file )!=']' ) assert( 0 );
   fclose( file );
}



