/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

typedef BYTE * BITVECTOR;
#define BV_BLOCK 8
#define BV_WORD_SIZE 4

bool BV_IS_SET( BITVECTOR variable, UINT64 target )
{
   BITVECTOR addr = variable + target / BV_BLOCK;
   BYTE val = 1 << ( BV_BLOCK - ( target % BV_BLOCK ) - 1 );
   return *addr & val;
}
void BV_SET( BITVECTOR variable, UINT64 target )
{
   BITVECTOR addr = variable + target / BV_BLOCK;
   BYTE val = 1 << ( BV_BLOCK - ( target % BV_BLOCK ) - 1 );
   *addr |= val;
}
void BV_CLEAR( BITVECTOR variable, UINT64 target )
{
   BITVECTOR addr = variable + target / BV_BLOCK;
   BYTE val = 1 << ( BV_BLOCK - ( target % BV_BLOCK ) - 1 );
   *addr &= ~val;
}
void BV_TOGGLE( BITVECTOR variable, UINT64 target )
{
   BITVECTOR addr = variable + target / BV_BLOCK;
   BYTE val = 1 << ( BV_BLOCK - ( target % BV_BLOCK ) - 1 );
   *addr ^= val;
}
UINT64 BV_SIZE( UINT64 bits )
{
   if( bits % ( BV_BLOCK * BV_WORD_SIZE ) )
      return ( bits / ( BV_BLOCK * BV_WORD_SIZE ) + 1 ) * BV_WORD_SIZE;
   else return bits / BV_BLOCK;
}


