/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include <string.h>
#include <errno.h>
#include <stdio.h>

#ifndef TIVCHECK_H_
#define TIVCHECK_H_

#define malloc_check( mc_target, mc_type, mc_size )                            \
   {                                                                           \
      mc_target = ( mc_type )malloc( mc_size );                                \
      assert( mc_target );                                                     \
   }
   
#define realloc_check( target, type, size )                                    \
   {                                                                           \
      target = ( type )realloc( target, size );                                \
      assert( target );                                                        \
   }

#define fopen_check( stream, filename, perms )                              \
   if( !( stream = fopen( filename, perms ) ) )                             \
   {                                                                        \
      printf( "Error opening %s with permissions %s: %s\n",           \
              filename, perms, strerror( errno ) );                 \
      assert( 0 ); /* Failed to open file */                                \
   }
   
#define fread_check( target, size, stream )                                 \
   if( fread( target, 1, size, stream )!=size )                             \
   {                                                                        \
      printf( "Error reading stream %d: %s\n",                        \
              stream, strerror( errno ) );                          \
      assert( 0 ); /* Failed to read file */                                \
   }
   
void fwrite_check( void *target, UINT32 size, FILE *stream )
{
   if( fwrite( target, 1, size, stream )!=size )
   {                                                                        
      printf( "Error writing stream %d: %s\n", stream, strerror( errno ) );                          
      assert( 0 ); /* Failed to write file */                               
   }
}

#endif
