/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include <stdio.h>
#include <ctype.h>

#ifndef MAX_STRING_LENGTH
#define MAX_STRING_LENGTH 4098
#endif

// **NOTES**   You may only call one fread_string at a time (this
//             includes fread_line and word as well).  Strings are 
//             stored in a buffer, so it must be processed before
//             you can get another one.

static char pc_buffer[ MAX_STRING_LENGTH ];

static void fread_to_eol( FILE *pf_file )
{
   char c_char;

   do
   {
      c_char = getc( pf_file );
   }
   while ( c_char != '\n' && c_char != '\r' );

   do
   {
      c_char = getc( pf_file );
   }
   while ( c_char == '\n' || c_char == '\r' );

   ungetc( c_char, pf_file );
   return;
}

static long fread_number( FILE *pf_file )
{
   char c_char;
   long l_number = 0;
   bool b_sign = false;

   do
   {
      c_char = getc( pf_file );
   }
   while ( isspace(c_char) );

   if ( c_char == '+' )
   {
      c_char = getc( pf_file );
   }
   else if ( c_char == '-' )
   {
      b_sign = true;
      c_char = getc( pf_file );
   }

   if ( !isdigit( c_char ) ) return 0;

   while ( isdigit( c_char ) )
   {
      l_number = l_number * 10 + c_char - '0';
      c_char = getc( pf_file );
   }

   if ( b_sign ) l_number *= -1;

   if ( !isspace( c_char ) )
   ungetc( c_char, pf_file );

   return l_number;
}

static char *fread_string( FILE *pf_file, char c_end )
{
   char *pc_ptr = pc_buffer;
   char  c_char;

   do
   {
      c_char = getc( pf_file );
   }
   while( isspace( c_char ) && c_char!=c_end );

   ungetc( c_char, pf_file );

   while( 1 )
   {
      *pc_ptr = getc( pf_file );

      if( *pc_ptr==c_end )
      {
         *pc_ptr = '\0';
         return pc_buffer;
      }

      switch ( *pc_ptr )
      {
      default:
         pc_ptr++;
         break;
      case EOF:
         pc_buffer[0] = '\0';
         return pc_buffer;
         break;
      case '\n':
         pc_ptr++;
         *pc_ptr++ = '\r';
         break;
      case '\r':
         break;
      }

      if( pc_ptr - pc_buffer >= MAX_STRING_LENGTH - 1 )
      {
         *pc_ptr = '\0';
         return pc_buffer;
      }
   }
}

static char *fread_line( FILE *pf_file )
{
   char *pc_ptr = pc_buffer;
   char  c_char;

   while( 1 )
   {
      switch ( *pc_ptr = getc( pf_file ) )
      {
      default:
         pc_ptr++;
         break;
      case EOF:
         pc_buffer[0] = '\0';
         return pc_buffer;
         break;
      case '\n':
         *pc_ptr = '\0';
         do
         {
            c_char = getc( pf_file );
         }
         while( c_char == '\n' || c_char == '\r' );

         ungetc( c_char, pf_file );

         return pc_buffer;
         break;
      }

      if( pc_ptr - pc_buffer >= MAX_STRING_LENGTH - 1 )
      {
         *pc_ptr = '\0';
         return pc_buffer;
      }
   }
}

static char *fread_word( FILE *pf_file )
{
   char *pc_ptr = pc_buffer;
   char  c_char;
   char  c_end_char;

   do
   {
      c_char = getc( pf_file );
   }
   while( isspace( c_char ) );

   if ( c_char == '\'' || c_char == '"' )
      c_end_char = c_char;
   else
   {
      c_end_char = '\0';
      ungetc( c_char, pf_file );
   }

   while( 1 )
   {
      switch ( *pc_ptr = getc( pf_file ) )
      {
      default:
         if( c_end_char && *pc_ptr == c_end_char )
         {
            *pc_ptr = '\0';
            return pc_buffer;
         }
         else if( !c_end_char && isspace( *pc_ptr ) )
         {
            *pc_ptr = '\0';
            return pc_buffer;
         }
         pc_ptr++;
         break;
      case EOF:
         pc_buffer[0] = '\0';
         return pc_buffer;
         break;
      }

      if( pc_ptr - pc_buffer >= MAX_STRING_LENGTH - 1 )
      {
         *pc_ptr = '\0';
         return pc_buffer;
      }
   }
}

static char fread_letter( FILE *pf_file )
{
   char c_char;

   do
   {
      c_char = getc( pf_file );
   }
   while ( isspace( c_char ) );

   return c_char;
}


