/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#ifndef TIVHASH
#define TIVHASH

#include "tivvect.h"
#include <assert.h>
#include <iostream>
using namespace std;

template <class T> class Hash  {
public:
   // Default Constructor - Initialize with length 0 and no buffer
   Hash( UINT32 mySize, UINT32 (*slotFn)(T value), int (*cmpFn)(T left, T right) )
   {
      data = new vector <T> [mySize];
      size = mySize;
      slot = slotFn;
      compare = cmpFn;
      assert( data );
   }
   
   // Destructor - free the buffer
   ~Hash()
   {
      delete [] data;
   }

   void empty( )
   {
      for( UINT32 count = 0; count < size; count++ )
         data[count].resize( 0 );
   }
         

   // Add an element to the hash if it's not already there
   bool add(T value) const
   {
      UINT32 mySlot = (*slot)( value ) % size;
      for( UINT32 count = 0; count < data[mySlot].size(); count++ )
         if( !(*compare)( data[mySlot][count], value ) ) return false;
      data[mySlot].push_back( value );
      return true;
   }
private:
   vector <T> *data;
   UINT32 size;
   UINT32 (*slot)(T value);
   int (*compare)(T left, T right);
};

#endif
