/**************************************************************************
 *  enumerator - a program to enumerate large implicit search spaces      *
 *  Copyright (C) 2007-2008  Eric Robinson                                *
 *                                                                        *
 *  This program is free software: you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation, either version 3 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include <assert.h>

#define BYTE_LENGTH        8

// Standard definitions for both windows and unix machine
#ifdef WIN32
   // Special types
   typedef int                BOOL;
   typedef unsigned char      BYTE;
   typedef char               CHAR;

   
   // The normal ones
   typedef float              FLOAT32;
   typedef double             FLOAT64;
   typedef char               INT1;
   typedef char               INT8;
   typedef short              INT16;
   typedef int                INT32;
   typedef __int64            INT64;
   
   // Their unsigned versions
   typedef unsigned char      UINT1;
   typedef unsigned char      UINT8;
   typedef unsigned short     UINT16;
   typedef unsigned int       UINT32;
   typedef unsigned __int64   UINT64;

   // Their lengths
   #define SIZEBOOL           sizeof( BOOL )
   #define SIZE1              sizeof( INT1 )
   #define SIZEBYTE           sizeof( BYTE )
   #define SIZECHAR           sizeof( CHAR )
   #define SIZE8              sizeof( INT8 )
   #define SIZE16             sizeof( INT16 )
   #define SIZE32             sizeof( INT32 )
   #define SIZE64             sizeof( INT64 )
#else

   // Special types
   typedef unsigned char      BOOL;
   typedef unsigned char      BYTE;
   typedef char               CHAR;

   
   // The normal ones
   typedef float              FLOAT32;
   typedef double             FLOAT64;
   typedef char               INT1;
   typedef char               INT8;
   typedef short              INT16;
   typedef int                INT32;
   typedef long long          INT64;
   
   // Their unsigned versions
   typedef unsigned char      UINT1;
   typedef unsigned char      UINT8;
   typedef unsigned short     UINT16;
   typedef unsigned int       UINT32;
   typedef unsigned long long UINT64;

   // Their lengths
   #define SIZEBOOL           sizeof( BOOL )
   #define SIZE1              sizeof( INT1 )
   #define SIZEBYTE           sizeof( BYTE )
   #define SIZECHAR           sizeof( CHAR )
   #define SIZE8              sizeof( INT8 )
   #define SIZE16             sizeof( INT16 )
   #define SIZE32             sizeof( INT32 )
   #define SIZE64             sizeof( INT64 )
#endif
