#include "mpi.h"
#include "mpiimpl.h"

int MPI_Alltoall (
     void *sendbuf,
     int sendcount,
     MPI_Datatype sendtype,
     void *recvbuf,
     int recvcount,
     MPI_Datatype recvtype,
     MPI_Comm comm)
{
  int dest_rank;
  MPI_Status status;
  
  /* each process sends what's in their sendbuf to everybody else */
  for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
    char *sb = (char *)sendbuf;
    sb += (dest_rank*MPINU_msg_size(sendcount,sendtype));
    MPINU_Int_Send(sb, sendcount, sendtype, dest_rank, MPINU_ALLTOALL_TAG,
	comm);
  }

  /* each process receives from all other processes; the jth element
     sent by process i to process j is saved in the ith position in
     the recvbuf */
  for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
    char *rb = (char *)recvbuf;
    rb += (dest_rank*MPINU_msg_size(recvcount,recvtype));
    MPINU_Int_Recv(rb, recvcount, recvtype, dest_rank, MPINU_ALLTOALL_TAG,
	comm, &status);
  }

  return MPI_SUCCESS;
}
