#include "mpi.h"
#include "mpiimpl.h"

int MPI_Alltoallv (
     void *sendbuf,
     int *sendcounts,
     int *sdispls,
     MPI_Datatype sendtype,
     void *recvbuf,
     int *recvcounts,
     int* rdispls,
     MPI_Datatype recvtype,
     MPI_Comm comm)
{
  int dest_rank;
  MPI_Status status;

  /* each process sends what's in their sendbuf to everybody else; the
     size of data sent can vary among processes */
  for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
    char *sb = (char *)sendbuf;
    sb += (sdispls[dest_rank]*MPINU_datatype_size(sendtype));
    MPINU_Int_Send(sb, sendcounts[dest_rank], sendtype, dest_rank,
	MPINU_ALLTOALLV_TAG, comm);
  }

  /* each process receives from all other processes; the jth element
     sent by process i to process j is saved in the ith position in
     the recvbuf */
  for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
    char *rb = (char *)recvbuf;
    rb += (rdispls[dest_rank]*MPINU_datatype_size(recvtype));
    MPINU_Int_Recv(rb, recvcounts[dest_rank], recvtype, dest_rank,
	MPINU_ALLTOALLV_TAG, comm, &status);
  }
 
  return MPI_SUCCESS;
}
