#include "mpi.h"
#include "mpiimpl.h"

int MPI_Bcast (
    void             *buffer,
    int               count,
    MPI_Datatype      datatype,
    int               root,
    MPI_Comm          comm)
{
  int dest_rank;
  MPI_Status status;

  /* a broadcast implementation using a binary tree, where root is the root of
   the tree, its left child is dest_rank 1, and right child is dest_rank 2 (if
   they exist) and so on. Every left child is odd and every right child is
   even */

  /* all processes except for root receives data from a parent node */
  if (MPINU_myrank != root)
  {
    dest_rank = (int)(ceil((double)MPINU_myrank/(double)2)) - 1;
    MPINU_Int_Recv(buffer, count, datatype, dest_rank, MPINU_BCAST_TAG, comm,
	&status);
  }

  /* all processes including root, sends data to a left and right
     child if they exist */
  dest_rank = (2*MPINU_myrank)+1 ; /* left child */
  if (dest_rank < (MPINU_num_slaves + 1))
    MPINU_Int_Send(buffer, count, datatype, dest_rank, MPINU_BCAST_TAG, comm);
  dest_rank = (2*MPINU_myrank)+2 ; /* right child */
  if (dest_rank < (MPINU_num_slaves + 1))
    MPINU_Int_Send(buffer, count, datatype, dest_rank, MPINU_BCAST_TAG, comm);

  return MPI_SUCCESS;
}
