#ifndef _CDLL_H_
#define _CDLL_H_

/*
 * Interface for a circularly-doubly-linked-list package.
 *
 * These types carry no payload, so they are meant to be embedded in the data
 * structure proper.
 */

typedef struct cdll_header_tag {
  void *h0;
  void *h1;
  void *h2;
} cdll_header_t, *cdll_header_p;

typedef struct cdll_element_tag {
  void *e0;
  void *e1;
  void *e2;
} cdll_element_t, *cdll_element_p;

/*
 * Initialize and destroy a previously allocated header.
 */
extern void cdll_header_init( cdll_header_p );
extern void cdll_header_destroy( cdll_header_p );

/*
 * Given a header, is the list empty or not?
 */
extern int cdll_empty( cdll_header_p );

/*
 * Insert the given element at the beginning of the list.
 */
extern void cdll_insert_head( cdll_header_p, cdll_element_p );

/*
 * Insert the given element at the end of the list.
 */
extern void cdll_insert_tail( cdll_header_p, cdll_element_p );

/*
 * Remove an element from the head of the list, returning NULL if the list is
 * empty.
 */
extern cdll_element_p cdll_dequeue_head( cdll_header_p );

/*
 * Remove an element from the tail of the list, returning NULL if the list is
 * empty.
 */
extern cdll_element_p cdll_dequeue_tail( cdll_header_p );

/*
 * Return a pointer to the element at the head of the list.
 */
extern cdll_element_p cdll_head( cdll_header_p );

/*
 * Return a pointer to the element at the head of the list.
 */
extern cdll_element_p cdll_tail( cdll_header_p );

/*
 * Initialize and destroy a previously allocated element.
 */
extern void cdll_element_init( cdll_element_p );
extern void cdll_element_destroy( cdll_element_p );

/*
 * Remove the given element from the list on which it resides.
 */
extern cdll_element_p cdll_remove( cdll_element_p );

/*
 * Return the next element in the list. Note that if this is the last element
 * in the list, the next element returned will be the header; the caller must
 * check for this.
 */
extern cdll_element_p cdll_next( cdll_element_p );

/*
 * Return the previous element in the list. Note that if this is the first
 * element in the list, the previous element returned will be the header; the
 * caller must check for this.
 */
extern cdll_element_p cdll_prev( cdll_element_p );

/*
 * Insert the second element before the first.
 */
extern void cdll_insert_before( cdll_element_p, cdll_element_p );

/*
 * Insert the second element after the first.
 */
extern void cdll_insert_after( cdll_element_p, cdll_element_p );

#endif /* _CDLL_H_ */
