#ifndef _CACHE_H_
#define _CACHE_H_

/*
 * Cache interface used by MPInu. The cache does not know about the MPI
 * implementation, it just provides a set of methods that provide the
 * following semantics:
 *
 * A. Elements are kept sorted in the cache by time of entry.
 * B. Elements are search from oldest to newest.
 */

/*
 * Opaque definition of a cache object.
 */
typedef void *buf_t, **buf_p;

/*
 * Opaque definition of an object stored in the cache.
 */
typedef void *buf_obj_t, **buf_obj_p;

/*
 * Opaque definition of the key associated with an object.
 */
typedef void *buf_key_t;

/*
 * Initialize a cache buffer, with the routine to be called to compare objects
 * and keys. Returns zero on success, non-zero on failure.
 */
int buf_init (buf_p, int (*)(buf_key_t, buf_key_t));

/*
 * Associate a key with an object. Returns zero on success, non-zero on failure.
 */
int buf_put (buf_t, buf_key_t, buf_obj_t);

/*
 * Lookup a kay in the cache; returns TRUE if found, FALSE otherwise.
 */
int buf_lookup (buf_t, buf_key_t);

/*
 * Pull a buffer from the cache matching the given key.
 *
 * If key is found and object is returned, then object must be the oldest
 * cache element of the cache having that key.
 */
int buf_get (buf_t, buf_key_t, buf_obj_p);

/*
 * Tear down a buffer cache, releasing any and all resources used to maintain
 * the cache. Use the provide free routine to release sources associated with
 * the objects stored in the cache.
 *
 * Return zero on success, non-zero on failure.
 */
int buf_destroy (buf_t,  void (*)(buf_obj_t));

#endif
