/* This is public portion of mpi.h */

/* Keep C++ compilers from getting confused */
#if defined(__cplusplus)
extern "C" {
#endif

/*
 * MPInu Version number: JJJ-NNN-PPP-BBB-UUU
 *
 * Where: JJJ - Major number
 *	  NNN - Minor number
 *	  PPP - Patch number
 *	  BBB - Beta number
 *	  UUU - Build number
 */
#define MPINU_VERSION	002005001007001LL

typedef int MPI_Comm;
typedef struct { int MPI_SOURCE;
		 int MPI_TAG;
		 int mpi_size;
	       } MPI_Status;
#define MPI_STATUS_IGNORE ((MPI_Status *)0)

#define MPI_SUCCESS 0
#define MPI_FAIL 1

#define MPI_COMM_WORLD 0
#define MPI_ANY_TAG -1
#define MPI_ANY_SOURCE -1

#define MPI_HOST 1
#define MPI_TAG_UB 2
#define MPI_IO 3
#define MPI_WTIME_IS_GLOBAL 4
#define MPI_PROC_NULL 0

/* Datatypes:  */
/* These must correspond with datatype_size[] in sendrecv.c */
enum MPINU_Datatype {
	MPI_MIN_DATATYPE = 0,
	MPI_CHAR = 1,
	MPI_UNSIGNED_CHAR,
	MPI_BYTE,
	MPI_SHORT,
	MPI_UNSIGNED_SHORT,
	MPI_INT,
	MPI_UNSIGNED,
	MPI_LONG,
	MPI_UNSIGNED_LONG,
	MPI_FLOAT,
	MPI_DOUBLE,
	MPI_LONG_DOUBLE,
	MPI_LONG_LONG_INT,
	MPI_SHORT_INT,
	MPI_INT_INT,
	MPI_LONG_INT,
	MPI_FLOAT_INT,
	MPI_DOUBLE_INT,
	MPI_LONG_DOUBLE_INT,
	MPI_MAX_DATATYPE
};
typedef enum MPINU_Datatype MPI_Datatype;

/* Reduce Operations:  */
typedef int MPI_Op;

#define MPI_MAX           ((MPI_Op)1)
#define MPI_MIN           ((MPI_Op)2)
#define MPI_PROD          ((MPI_Op)3)
#define MPI_SUM           ((MPI_Op)4)
#define MPI_LAND          ((MPI_Op)5)
#define MPI_LOR           ((MPI_Op)6)
#define MPI_LXOR          ((MPI_Op)7)
#define MPI_BAND          ((MPI_Op)8)
#define MPI_BOR           ((MPI_Op)9)
#define MPI_BXOR          ((MPI_Op)10)
#define MPI_MAXLOC        ((MPI_Op)11)
#define MPI_MINLOC        ((MPI_Op)12)

// By definition, the values must only be monotonically increasing.
#define MPI_THREAD_SINGLE	0x10
#define MPI_THREAD_FUNNELED	0x20
#define MPI_THREAD_SERIALIZED	0x30
#define MPI_THREAD_MULTIPLE	0x40

int MPI_Abort(MPI_Comm comm, int errorcode);
int MPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
int MPI_Comm_rank(MPI_Comm comm, int *rank);
int MPI_Comm_size(MPI_Comm comm, int *size);
#define MPI_MAX_ERROR_STRING 80
int MPI_Error_string(int errorcode, char *string, int *resultlen);
int MPI_Finalize(void);
int MPI_Get_count(MPI_Status *status, MPI_Datatype datatype, int *count);
int MPI_Get_processor_name(char *name, int *resultlen);
int MPI_Init(int *argc_ptr, char ***argv_ptr);
int MPI_Init_thread(int *argc_ptr, char ***argv_ptr, int required, int *provided);
int MPI_Initialized(int *flag);
int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status
);
int MPI_Is_thread_main(int *flag);
int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);
int MPI_Query_thread(int *provided);
int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
int MPI_Send(void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

#if defined(__cplusplus)
}
#endif
