#include "mpi.h"
#include "mpiimpl.h"

int MPI_Reduce_scatter (
     void *sendbuf,
     void *recvbuf,
     int *recvcounts,
     MPI_Datatype datatype,
     MPI_Op op,
     MPI_Comm comm)
{
  int count, dest_rank;
  int datasize, i;
  MPI_Status status;
  int *displs;
  void *tempbuf;

  count=0;

  /* count the total number elements that need to be reduced */
  for (i=0; i<=MPINU_num_slaves; i++)
    count=count+recvcounts[i];

  if (MPINU_myrank == 0)
    {
      displs = (int *)malloc((MPINU_num_slaves+1)*sizeof(int));
      assert( displs );
      displs[0] = 0;
      for (i=1; i<=MPINU_num_slaves; i++)
	displs[i]=recvcounts[i-1]+displs[i-1];
    }

  /* apply reduction operation; result will be in recvbuf */
  MPI_Reduce(sendbuf, recvbuf, count, datatype, op, 0, comm);

  /* result is scattered among the processes */
  if (MPINU_myrank == 0)
    {
      /* copy recvbuf into tempbuf and use tempbuf to scatter the result
	 among the nodes */
      int size = MPINU_msg_size(count,datatype);
      tempbuf = malloc(size);
      assert( tempbuf );
      memcpy(tempbuf, recvbuf, size);
      MPI_Scatterv(tempbuf, recvcounts, displs, datatype, recvbuf,
         recvcounts[MPINU_myrank], datatype, 0, MPI_COMM_WORLD);  
    }
  else
    MPI_Scatterv(sendbuf, recvcounts, displs, datatype, recvbuf,
		 recvcounts[MPINU_myrank], datatype, 0, MPI_COMM_WORLD);

  return MPI_SUCCESS;
}
