#include "mpi.h"
#include "mpiimpl.h"

int MPI_Reduce (
     void *sendbuf,
     void *recvbuf,
     int count,
     MPI_Datatype datatype,
     MPI_Op op,
     int root,
     MPI_Comm comm)
{
  int i;
  int dest_rank;
  size_t datasize;
  MPI_Status status;
  void *tempbuf;

  if (MPINU_myrank == root)
    {
      datasize = MPINU_msg_size(count,datatype);
      tempbuf=malloc(datasize);
      assert( tempbuf );
  
      /* if starting at root, copy values in sendbuf to recvbuf */
      memcpy(recvbuf, sendbuf, datasize);

      /*  perform reduction on root's recvbuf and values sent by rest of 
          processes to root */
      for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ )
        {
          if ( dest_rank != root )
            {
              MPINU_Int_Recv(tempbuf, count, datatype, dest_rank,
		  MPINU_REDUCE_TAG, comm, &status);
              /* call op-function on tempbuf(other process's sendbuf) and
                 root's recvbuf */ 
	      exec_op(tempbuf, recvbuf, count, datatype, op);
	    }
        }
    }
  else
    MPINU_Int_Send(sendbuf, count, datatype, root, MPINU_REDUCE_TAG, comm);

  return MPI_SUCCESS;
}
