#include "mpi.h"
#include "mpiimpl.h"

int MPI_Scan (
     void *sendbuf,
     void *recvbuf,
     int count,
     MPI_Datatype datatype,
     MPI_Op op,
     MPI_Comm comm)
{
  /* this is an inclusive scan, so the reduction on process i includes data
     from process i;  this implementation is based on the algorithm
     provided in the mpich mpi_scan implementation; this process is
     also known as parallel prefix */
  
  size_t datasize;
  int i;
  unsigned int mask, dest;
  void *partscan;
  void *tempbuf;
  MPI_Status status;

  datasize = MPINU_msg_size(count,datatype);
  partscan = (void *)malloc(datasize);
  assert( partscan );
  tempbuf = (void *)malloc(datasize);
  assert( tempbuf );

  /* if starting at root, copy values in sendbuf to recvbuf */
  memcpy(recvbuf, sendbuf, datasize);
  memcpy(partscan, sendbuf, datasize);

  mask = 0x1;
  
  while ( mask < MPINU_num_slaves+1 )
    {
      dest = MPINU_myrank^mask;
      /* make sure we're only sending to process that exists */
      if ( dest < MPINU_num_slaves+1 )
        {
	  /* send partial result to process 'mask' distance away */
          MPINU_Int_Send(partscan, count, datatype, dest, MPINU_SCAN_TAG, comm);
	  /* receive from that process */
          MPINU_Int_Recv(tempbuf, count, datatype, dest, MPINU_SCAN_TAG, comm,
	      &status);

	  /* update data by doing op */
	  if ( MPINU_myrank > dest )
            {
	      exec_op(tempbuf, partscan, count, datatype, op);
	      exec_op(tempbuf, recvbuf, count, datatype, op);
            }
          else
	    exec_op(tempbuf, partscan, count, datatype, op);
	}
      mask <<= 1;
    }
  return MPI_SUCCESS;
}
