#include "mpi.h"
#include "mpiimpl.h"

int MPI_Scatter (
     void              *sendbuf,
     int               sendcount,
     MPI_Datatype      sendtype,
     void              *recvbuf,
     int               recvcount,
     MPI_Datatype      recvtype,
     int               root,
     MPI_Comm          comm)
{
  int dest_rank;
  MPI_Status status;

  /* if root, send contents of sendbuf to other processes */
  if (MPINU_myrank == root)
    {
      for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
	char *sb = (char *)sendbuf;
	sb += (dest_rank*MPINU_msg_size(sendcount,sendtype));
        MPINU_Int_Send(sb, sendcount, sendtype, dest_rank, MPINU_SCATTER_TAG,
	    comm);
      }
    }

  /* recv the piece of data from root and store it in recvbuf */
  return MPINU_Int_Recv(recvbuf, recvcount, recvtype, root, MPINU_SCATTER_TAG,
      comm, &status);
}
