#include "mpi.h"
#include "mpiimpl.h"


int MPI_Scatterv (
     void *sendbuf,
     int *sendcnts,
     int *displs,
     MPI_Datatype sendtype,
     void *recvbuf,
     int recvcnt,
     MPI_Datatype recvtype,
     int root,
     MPI_Comm comm)
{
  int dest_rank;
  MPI_Status status;

  /* if root, send contents of sendbuf to other processes */
  if (MPINU_myrank == root)
    {
      for ( dest_rank = 0; dest_rank <= MPINU_num_slaves; dest_rank++ ) {
	char *sb = (char *)sendbuf;
	sb += (displs[dest_rank]*MPINU_datatype_size(sendtype));
        MPINU_Int_Send(sb, sendcnts[dest_rank], sendtype, dest_rank,
	    MPINU_SCATTERV_TAG, comm);
      }
    }

  /* recv the piece of data from root and store it in recvbuf */
  return MPINU_Int_Recv(recvbuf, recvcnt, recvtype, root, MPINU_SCATTERV_TAG,
      comm, &status);
}
