#ifndef _SIGNAL_HANDLING_H_
#define _SIGNAL_HANDLING_H_

 /**********************************************************************
  * MPINU				                               *
  * Copyright (c) 2004-2005 Gene Cooperman <gene@ccs.neu.edu>          *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact Gene Cooperman          *
  * <gene@ccs.neu.edu>.                                                *
  **********************************************************************/

/*
 * Signal handlers and threads don't mix well. We avoid the very difficult
 * problems the arise trying to debug threaded programs using signal handlers
 * by create a facility that uses a sigwait thread to service objects that
 * describe actions to be taken when a signal occurs.
 */

/*
 * An opaque data type for the (s)ignal (h)andler (o)bject.
 */
typedef void	*sho_t;

/*
 * Block all asynchronous signals.
 */
void sho_block_async_signals (sigset_t *);

/*
 * Restore previously saved signal mask.
 */
void sho_restore_async_signals (sigset_t *);

/*
 * Initialize the signal handling subsystem.
 */
int sho_init (void);

/*
 * Allocate a sho, given a signal number, a function and an argument for that
 * function.
 */
sho_t sho_alloc (int, void (*)(int, void *), void *);

/*
 * Activate the sho so that it will start handling signals.
 */
int sho_activate (sho_t);

/*
 * Deactivate the sho so that signals won't be handled any longer.
 */
int sho_deactivate (sho_t);

/*
 * Free the sho and any memory it might have allocated to it.
 */
void sho_free (sho_t);

#endif
