#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* general functionality test for MPI_Allgatherv */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */

  int ntasks;
  int *displs, *counts;
  int *meddispls, *medcounts;
  int *lrgdispls, *lrgcounts;
  int totalcnts, totalmedcnts, totallrgcnts;
  int i;
  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
                &ntasks);

  /* test datatypes: char, int, unsigned int and double */

  char *buffer1;
  char *recv_buffer1;
  int *buffer2;
  int *recv_buffer2;
  unsigned int *buffer3;
  unsigned int *recv_buffer3;
  double *buffer4;
  double *recv_buffer4;

  /* a few of each datatype */
  char *medbuffer1;
  char *recv_medbuffer1;
  int *medbuffer2;
  int *recv_medbuffer2;
  unsigned int *medbuffer3;
  unsigned int *recv_medbuffer3;
  double *medbuffer4;
  double *recv_medbuffer4;

  /* larger number of each datatype */
  char *lrgbuffer1;
  char *recv_lrgbuffer1;
  int *lrgbuffer2;
  int *recv_lrgbuffer2;
  unsigned int *lrgbuffer3;
  unsigned int *recv_lrgbuffer3;
  double *lrgbuffer4;
  double *recv_lrgbuffer4;

  /* allocate counts and displacements */
  counts=(int*)malloc(ntasks*sizeof(int));
  displs=(int*)malloc(ntasks*sizeof(int));
  medcounts=(int*)malloc(ntasks*sizeof(int));
  meddispls=(int*)malloc(ntasks*sizeof(int));
  lrgcounts=(int*)malloc(ntasks*sizeof(int));
  lrgdispls=(int*)malloc(ntasks*sizeof(int));
  
  /* initialiaze counts */
  totalcnts = 0;
  totalmedcnts = 0;
  totallrgcnts = 0;
  for (i=0; i<ntasks; i++)
    {
      counts[i]=i+1;
      totalcnts += counts[i];
      medcounts[i]=(i+1)*512;
      totalmedcnts += medcounts[i];
      lrgcounts[i]=(i+1)*12800;
      totallrgcnts += lrgcounts[i];
    }

  /* initialiaze displacements */
  displs[0]=0;
  meddispls[0]=0;
  lrgdispls[0]=0;
  for ( i=1; i<ntasks; i++ )
    {
      displs[i]=counts[i-1]+displs[i-1];
      meddispls[i]=medcounts[i-1]+meddispls[i-1];
      lrgdispls[i]=lrgcounts[i-1]+lrgdispls[i-1];
    }
  
  /* allocate send and recv buffers */
  buffer1 = (char *)malloc(totalcnts*sizeof(char));
  recv_buffer1 = (char *)malloc(totalcnts*sizeof(char));

  buffer2 = (int *)malloc(totalcnts*sizeof(int)); 
  recv_buffer2 = (int *)malloc(totalcnts*sizeof(int)); 

  buffer3 = (unsigned int *)malloc(totalcnts*sizeof(unsigned int));
  recv_buffer3 = (unsigned int *)malloc(totalcnts*sizeof(unsigned int));

  buffer4 = (double *)malloc(totalcnts*sizeof(double));
  recv_buffer4 = (double *)malloc(totalcnts*sizeof(double));

  medbuffer1 = (char *)malloc(totalmedcnts*sizeof(char));
  recv_medbuffer1 = (char *)malloc(totalmedcnts*sizeof(char));

  medbuffer2 = (int *)malloc(totalmedcnts*sizeof(int));
  recv_medbuffer2 = (int *)malloc(totalmedcnts*sizeof(int));

  medbuffer3 = (unsigned int *)malloc(totalmedcnts*sizeof(unsigned int));
  recv_medbuffer3 = (unsigned int *)malloc(totalmedcnts*sizeof(unsigned int));

  medbuffer4 = (double *)malloc(totalmedcnts*sizeof(double));
  recv_medbuffer4 = (double *)malloc(totalmedcnts*sizeof(double)); 

  lrgbuffer1 = (char *)malloc(totallrgcnts*sizeof(char));
  recv_lrgbuffer1 = (char *)malloc(totallrgcnts*sizeof(char));

  lrgbuffer2 = (int *)malloc(totallrgcnts*sizeof(int));
  recv_lrgbuffer2 = (int *)malloc(totallrgcnts*sizeof(int));

  lrgbuffer3 = (unsigned int *)malloc(totallrgcnts*sizeof(unsigned int));
  recv_lrgbuffer3 = (unsigned int *)malloc(totallrgcnts*sizeof(unsigned int));

  lrgbuffer4 = (double *)malloc(totallrgcnts*sizeof(double));
  recv_lrgbuffer4 = (double *)malloc(totallrgcnts*sizeof(double));

  /* initialize all send buffers */
  init_buffer(buffer1, totalcnts, TCHAR);
  init_buffer(buffer2, totalcnts, TINT);
  init_buffer(buffer3, totalcnts, TUINT);
  init_buffer(buffer4, totalcnts, TDOUBLE);

  init_buffer(medbuffer1, totalmedcnts, TCHAR);
  init_buffer(medbuffer2, totalmedcnts, TINT);
  init_buffer(medbuffer3, totalmedcnts, TUINT);
  init_buffer(medbuffer4, totalmedcnts, TDOUBLE);
 
  init_buffer(lrgbuffer1, totallrgcnts, TCHAR);
  init_buffer(lrgbuffer2, totallrgcnts, TINT);
  init_buffer(lrgbuffer3, totallrgcnts, TUINT);
  init_buffer(lrgbuffer4, totallrgcnts, TDOUBLE);

  MPI_Allgatherv(buffer1, counts[myrank], MPI_CHAR, recv_buffer1, counts,
		 displs, MPI_CHAR, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(buffer1, recv_buffer1+displs[i], counts[i]) != 0)
      printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Allgatherv(buffer2, counts[myrank], MPI_INT, recv_buffer2, counts,
              displs, MPI_INT, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(buffer2, recv_buffer2+displs[i], counts[i]) != 0)
      printf("rank: %d, buffer2 received incorrectly\n", myrank);

  MPI_Allgatherv(buffer3, counts[myrank], MPI_UNSIGNED, recv_buffer3, counts,
              displs, MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(buffer3, recv_buffer3+displs[i], counts[i]) != 0)
      printf("rank: %d, buffer3 received incorrectly\n", myrank);

  MPI_Allgatherv(buffer4, counts[myrank], MPI_DOUBLE, recv_buffer4, counts,
              displs, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(buffer4, recv_buffer4+displs[i], counts[i]) != 0)
      printf("rank: %d, buffer4 received incorrectly\n", myrank);

  MPI_Allgatherv(medbuffer1, medcounts[myrank], MPI_CHAR, recv_medbuffer1,
	      medcounts, meddispls, MPI_CHAR, 0, MPI_COMM_WORLD); 
  for(i=0; i<ntasks; i++)
    if (memcmp(medbuffer1, recv_medbuffer1+meddispls[i], medcounts[i]) != 0)
      printf("rank: %d, medbuffer1 received incorrectly\n", myrank);

  MPI_Allgatherv(medbuffer2, medcounts[myrank], MPI_INT, recv_medbuffer2,
	      medcounts, meddispls, MPI_INT, 0, MPI_COMM_WORLD); 
  for(i=0; i<ntasks; i++)
    if (memcmp(medbuffer2, recv_medbuffer2+meddispls[i], medcounts[i]) != 0)
      printf("rank: %d, medbuffer2 received incorrectly\n", myrank);

  MPI_Allgatherv(medbuffer3, medcounts[myrank], MPI_UNSIGNED, recv_medbuffer3,
	      medcounts, meddispls, MPI_UNSIGNED, 0, MPI_COMM_WORLD); 
  for(i=0; i<ntasks; i++)
    if (memcmp(medbuffer3, recv_medbuffer3+meddispls[i], medcounts[i]) != 0)
      printf("rank: %d, medbuffer3 received incorrectly\n", myrank);

  MPI_Allgatherv(medbuffer4, medcounts[myrank], MPI_DOUBLE, recv_medbuffer4,
	      medcounts, meddispls, MPI_DOUBLE, 0, MPI_COMM_WORLD); 
  for(i=0; i<ntasks; i++)
    if (memcmp(medbuffer4, recv_medbuffer4+meddispls[i], medcounts[i]) != 0)
      printf("rank: %d, medbuffer4 received incorrectly\n", myrank);

  MPI_Allgatherv(lrgbuffer1, lrgcounts[myrank], MPI_CHAR, recv_lrgbuffer1,
	      lrgcounts, lrgdispls, MPI_CHAR, 0, MPI_COMM_WORLD); 
  for(i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer1, recv_lrgbuffer1+lrgdispls[i], lrgcounts[i]) != 0)
      printf("rank: %d, lrgbuffer1 received incorrectly\n", myrank);

  MPI_Allgatherv(lrgbuffer2, lrgcounts[myrank], MPI_INT, recv_lrgbuffer2,
              lrgcounts, lrgdispls, MPI_INT, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer2, recv_lrgbuffer2+lrgdispls[i], lrgcounts[i])
	!= 0)
      printf("rank: %d, lrgbuffer2 received incorrectly\n", myrank);

  MPI_Allgatherv(lrgbuffer3, lrgcounts[myrank], MPI_UNSIGNED, recv_lrgbuffer3,
              lrgcounts, lrgdispls, MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer3, recv_lrgbuffer3+lrgdispls[i], lrgcounts[i])
	!= 0)
      printf("rank: %d, lrgbuffer3 received incorrectly\n", myrank);

  MPI_Allgatherv(lrgbuffer4, lrgcounts[myrank], MPI_DOUBLE, recv_lrgbuffer4,
		 lrgcounts, lrgdispls, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  for(i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer4, recv_lrgbuffer4+lrgdispls[i], lrgcounts[i])
	!= 0)
      printf("rank: %d, lrgbuffer4 received incorrectly\n", myrank);

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}
