#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* test for MPI_Allreduce on MPI_LXOR operation */

enum types { TCHAR, TINT, TUINT, TDOUBLE, TLONG };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  /* test datatype unsigned */
  /* test on one-element sendbuf */
  unsigned int *us = (unsigned int*)malloc(sizeof(unsigned int));
  unsigned int *ur = (unsigned int*)malloc(sizeof(unsigned int));
  /* test on many-element sendbuf */
  unsigned int *usendbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*ntasks);
  unsigned int *urecvbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*ntasks);
  /* test datatype long */
  /* test on one-element sendbuf */
  long *ds = (long*)malloc(sizeof(long));
  long *dr = (long*)malloc(sizeof(long));
  /* test on many-element sendbuf */
  long *dsendbuf = (long*)malloc(sizeof(long)*ntasks);
  long *drecvbuf = (long*)malloc(sizeof(long)*ntasks);
  
  us[0] = myrank;
  init_buffer(usendbuf, ntasks, TUINT);
  ds[0] = (long)myrank;
  init_buffer(dsendbuf, ntasks, TLONG);
  
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d us: ", myrank);
  printf("%d \n", us[0]);

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%ld ", dsendbuf[i]);
  printf("\n");

  printf("myrank: %d us: ", myrank);
  printf("%ld \n", ds[0]);


  MPI_Allreduce(usendbuf, urecvbuf, ntasks, MPI_UNSIGNED, MPI_LXOR, 0,
	     MPI_COMM_WORLD);
  MPI_Allreduce(dsendbuf, drecvbuf, ntasks, MPI_LONG, MPI_LXOR, 0,
	     MPI_COMM_WORLD);
  MPI_Allreduce(us, ur, 1, MPI_UNSIGNED, MPI_LXOR, 0, MPI_COMM_WORLD);
  MPI_Allreduce(ds, dr, 1, MPI_LONG, MPI_LXOR, 0, MPI_COMM_WORLD);

   
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%ld ", dsendbuf[i]);
  printf("\n");

  printf("urecvbuf ");
  for (i=0; i<ntasks; i++) 
    printf("%d ", urecvbuf[i]);
  printf("\n");
  printf("ur ");
  printf("%d \n", ur[0]);
  printf("drecvbuf ");
  for (i=0; i<ntasks; i++) 
    printf("%ld ", drecvbuf[i]);
  printf("\n");
  printf("dr ");
  printf("%ld \n", dr[0]);


  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;

    case TLONG:
      for(i=0; i<count; i++)
        ((long*)buf)[i]=(long)i+1;
      break;
    }
}
