#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* general functionality test for MPI_Alltoall */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */

  int ntasks;
  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
                &ntasks);
  int i;

  /* test datatypes: char, int, unsigned int and double */

  char *buffer1;
  char *recv_buffer1;
  int *buffer2;
  int *recv_buffer2;
  unsigned int *buffer3;
  unsigned int *recv_buffer3;
  double *buffer4;
  double *recv_buffer4;

  /* a few of each datatype */
  char *medbuffer1;
  char *recv_medbuffer1;
  int *medbuffer2;
  int *recv_medbuffer2;
  unsigned int *medbuffer3;
  unsigned int *recv_medbuffer3;
  double *medbuffer4;
  double *recv_medbuffer4;

  /* larger number of each datatype */
  char *lrgbuffer1;
  char *recv_lrgbuffer1;
  int *lrgbuffer2;
  int *recv_lrgbuffer2;
  unsigned int *lrgbuffer3;
  unsigned int *recv_lrgbuffer3;
  double *lrgbuffer4;
  double *recv_lrgbuffer4;

  int count = 1;
  int medcount = 1024;
  int lrgcount = 25600;

  /* allocate buffers */
  buffer1 = (char *)malloc(ntasks*count*sizeof(char));
  recv_buffer1 = (char *)malloc(ntasks*count*sizeof(char));

  buffer2 = (int *)malloc(ntasks*count*sizeof(int)); 
  recv_buffer2 = (int *)malloc(ntasks*sizeof(int)); 

  buffer3 = (unsigned int *)malloc(ntasks*count*sizeof(unsigned int));
  recv_buffer3 = (unsigned int *)malloc(ntasks*sizeof(unsigned int));

  buffer4 = (double *)malloc(ntasks*count*sizeof(double));
  recv_buffer4 = (double *)malloc(ntasks*sizeof(double));

  medbuffer1 = (char *)malloc(ntasks*medcount*sizeof(char));
  recv_medbuffer1 = (char *)malloc(ntasks*medcount*sizeof(char));

  medbuffer2 = (int *)malloc(ntasks*medcount*sizeof(int));
  recv_medbuffer2 = (int *)malloc(ntasks*medcount*sizeof(int));

  medbuffer3 = (unsigned int *)malloc(ntasks*medcount*sizeof(unsigned int));
  recv_medbuffer3 = (unsigned int *)malloc(ntasks*medcount*sizeof(unsigned int));

  medbuffer4 = (double *)malloc(ntasks*medcount*sizeof(double));
  recv_medbuffer4 = (double *)malloc(ntasks*medcount*sizeof(double)); 

  lrgbuffer1 = (char *)malloc(ntasks*lrgcount*sizeof(char));
  recv_lrgbuffer1 = (char *)malloc(ntasks*lrgcount*sizeof(char));

  lrgbuffer2 = (int *)malloc(ntasks*lrgcount*sizeof(int));
  recv_lrgbuffer2 = (int *)malloc(ntasks*lrgcount*sizeof(int));

  lrgbuffer3 = (unsigned int *)malloc(ntasks*lrgcount*sizeof(unsigned int));
  recv_lrgbuffer3 = (unsigned int *)malloc(ntasks*lrgcount*sizeof(unsigned int));

  lrgbuffer4 = (double *)malloc(ntasks*lrgcount*sizeof(double));
  recv_lrgbuffer4 = (double *)malloc(ntasks*lrgcount*sizeof(double));

  
  init_buffer(buffer1, ntasks*count, TCHAR);
  init_buffer(buffer2, ntasks*count, TINT);
  init_buffer(buffer3, ntasks*count, TUINT);
  init_buffer(buffer4, ntasks*count, TDOUBLE);

  init_buffer(medbuffer1, ntasks*medcount, TCHAR);
  init_buffer(medbuffer2, ntasks*medcount, TINT);
  init_buffer(medbuffer3, ntasks*medcount, TUINT);
  init_buffer(medbuffer4, ntasks*medcount, TDOUBLE);

  init_buffer(lrgbuffer1, ntasks*lrgcount, TCHAR);
  init_buffer(lrgbuffer2, ntasks*lrgcount, TINT);
  init_buffer(lrgbuffer3, ntasks*lrgcount, TUINT);
  init_buffer(lrgbuffer4, ntasks*lrgcount, TDOUBLE);
  
  MPI_Alltoall(buffer1, count, MPI_CHAR, recv_buffer1, count, MPI_CHAR, 0,
	      MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(buffer1+myrank, recv_buffer1+i, count) != 0)
      printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Alltoall(buffer2, count, MPI_INT, recv_buffer2, count, MPI_INT, 0,
	      MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(buffer2+myrank, recv_buffer2+i, count) != 0)
      printf("rank: %d, buffer2 received incorrectly\n", myrank);

  MPI_Alltoall(buffer3, count, MPI_UNSIGNED, recv_buffer3, count, MPI_UNSIGNED, 0,
	      MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(buffer3+myrank, recv_buffer3+i, count) != 0)
      printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Alltoall(buffer4, count, MPI_DOUBLE, recv_buffer4, count, MPI_DOUBLE, 0,
	      MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(buffer4+myrank, recv_buffer4+i, count) != 0)
      printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Alltoall(medbuffer1, medcount, MPI_CHAR, recv_medbuffer1, medcount, MPI_CHAR,
	       0, MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(medbuffer1+(myrank*medcount), recv_medbuffer1+(i*medcount), medcount) != 0)
      printf("rank: %d, medbuffer1 received incorrectly\n", myrank);

  MPI_Alltoall(medbuffer2, medcount, MPI_INT, recv_medbuffer2, medcount, MPI_INT,
	      0, MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(medbuffer2+(myrank*medcount), recv_medbuffer2+(i*medcount), medcount) != 0)
      printf("rank: %d, medbuffer2 received incorrectly\n", myrank);

  MPI_Alltoall(medbuffer3, medcount, MPI_UNSIGNED, recv_medbuffer3, medcount,
	       MPI_UNSIGNED, 0, MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(medbuffer3+(myrank*medcount), recv_medbuffer3+(i*medcount), medcount) != 0)
      printf("rank: %d, medbuffer3 received incorrectly\n", myrank);

  MPI_Alltoall(medbuffer4, medcount, MPI_DOUBLE, recv_medbuffer4, medcount,
	      MPI_DOUBLE, 0, MPI_COMM_WORLD);  
  for (i=0; i<ntasks; i++) 
    if (memcmp(medbuffer4+(myrank*medcount), recv_medbuffer4+(i*medcount), medcount) != 0)
      printf("rank: %d, medbuffer4 received incorrectly\n", myrank);

  MPI_Alltoall(lrgbuffer1, lrgcount, MPI_CHAR, recv_lrgbuffer1, lrgcount,
	      MPI_CHAR, 0, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++) 
    if (memcmp(lrgbuffer1+(myrank*lrgcount), recv_lrgbuffer1+(i*lrgcount),
	       lrgcount) != 0) 
      printf("rank: %d, lrgbuffer1 received incorrectly\n", myrank);

  MPI_Alltoall(lrgbuffer2, lrgcount, MPI_INT, recv_lrgbuffer2, lrgcount,
	      MPI_INT, 0, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++) 
    if (memcmp(lrgbuffer2+(myrank*lrgcount), recv_lrgbuffer2+(i*lrgcount),
	      lrgcount) != 0) 
      printf("rank: %d, lrgbuffer2 received incorrectly\n", myrank);

  MPI_Alltoall(lrgbuffer3, lrgcount, MPI_UNSIGNED, recv_lrgbuffer3, lrgcount,
              MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++) 
    if (memcmp(lrgbuffer3+(myrank*lrgcount), recv_lrgbuffer3+(i*lrgcount),
	       lrgcount) != 0) 
      printf("rank: %d, lrgbuffer3 received incorrectly\n", myrank);

  MPI_Alltoall(lrgbuffer4, lrgcount, MPI_DOUBLE, recv_lrgbuffer4, lrgcount,
	      MPI_DOUBLE, 0, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++) 
    if (memcmp(lrgbuffer4+(myrank*lrgcount), recv_lrgbuffer4+(i*lrgcount),
	      lrgcount) != 0) 
      printf("rank: %d, lrgbuffer4 received incorrectly\n", myrank);

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

void init_buffer(void *buf, int count, int type)
{
  int i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}
