#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* general functionality test for MPI_Alltoallv */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */

  int ntasks;
  int *sdispls, *rdispls, *scounts, *rcounts;
  int *medsdispls, *medrdispls, *medscounts, *medrcounts;
  int *lrgsdispls, *lrgrdispls, *lrgscounts, *lrgrcounts;
  int totalscnts, totalmedscnts, totallrgscnts;
  int totalrcnts, totalmedrcnts, totallrgrcnts;
  int i;
  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
                &ntasks);

  /* test datatypes: char, int, unsigned int and double */

  char *buffer1;
  char *recv_buffer1;
  int *buffer2;
  int *recv_buffer2;
  unsigned int *buffer3;
  unsigned int *recv_buffer3;
  double *buffer4;
  double *recv_buffer4;

  /* a few of each datatype */
  char *medbuffer1;
  char *recv_medbuffer1;
  int *medbuffer2;
  int *recv_medbuffer2;
  unsigned int *medbuffer3;
  unsigned int *recv_medbuffer3;
  double *medbuffer4;
  double *recv_medbuffer4;

  /* larger number of each datatype */
  char *lrgbuffer1;
  char *recv_lrgbuffer1;
  int *lrgbuffer2;
  int *recv_lrgbuffer2;
  unsigned int *lrgbuffer3;
  unsigned int *recv_lrgbuffer3;
  double *lrgbuffer4;
  double *recv_lrgbuffer4;

  /* allocate counts and displacements */
  scounts=(int*)malloc(ntasks*sizeof(int));
  rcounts=(int*)malloc(ntasks*sizeof(int));
  sdispls=(int*)malloc(ntasks*sizeof(int));
  rdispls=(int*)malloc(ntasks*sizeof(int));
  
  medscounts=(int*)malloc(ntasks*sizeof(int));
  medrcounts=(int*)malloc(ntasks*sizeof(int));
  medsdispls=(int*)malloc(ntasks*sizeof(int));
  medrdispls=(int*)malloc(ntasks*sizeof(int));

  lrgscounts=(int*)malloc(ntasks*sizeof(int));
  lrgrcounts=(int*)malloc(ntasks*sizeof(int));
  lrgsdispls=(int*)malloc(ntasks*sizeof(int));
  lrgrdispls=(int*)malloc(ntasks*sizeof(int));
  

  /* initialiaze counts */
  totalscnts = 0, totalrcnts = 0;
  totalmedscnts = 0, totalmedrcnts = 0;
  totallrgscnts = 0, totallrgrcnts = 0;
  for (i=0; i<ntasks; i++)
    {
      scounts[i]=i+1;
      totalscnts += scounts[i];
      rcounts[i]=myrank+1;
      totalrcnts += rcounts[i];
      medscounts[i]=(i+1)*512;
      totalmedscnts += medscounts[i];
      medrcounts[i]=(myrank+1)*512;
      totalmedrcnts += medrcounts[i];
      lrgscounts[i]=(i+1)*12800;
      totallrgscnts += lrgscounts[i];      
      lrgrcounts[i]=(myrank+1)*12800;
      totallrgrcnts += lrgrcounts[i];
    }

  /* initialiaze displacements */
  sdispls[0]=0;
  rdispls[0]=0;
  medsdispls[0]=0;
  medrdispls[0]=0;
  lrgsdispls[0]=0;
  lrgrdispls[0]=0;

  for ( i=1; i<ntasks; i++ )
    {
      sdispls[i]=scounts[i-1]+sdispls[i-1];
      rdispls[i]=i*(myrank+1);
      medsdispls[i]=medscounts[i-1]+medsdispls[i-1];
      medrdispls[i]=i*(myrank+1)*512;
      lrgsdispls[i]=lrgscounts[i-1]+lrgsdispls[i-1];
      lrgrdispls[i]=i*(myrank+1)*12800;
    }

  printf("myrank: %d, scounts ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", scounts[i]);
  printf("\n");

  printf("myrank: %d, rcounts ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", rcounts[i]);
  printf("\n");

  printf("myrank: %d, sdispls ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", sdispls[i]);
  printf("\n");

  printf("myrank: %d, rdispls ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", rdispls[i]);
  printf("\n");

  printf("myrank: %d, medscounts ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", medscounts[i]);
  printf("\n");

  printf("myrank: %d, medrcounts ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", medrcounts[i]);
  printf("\n");

  printf("myrank: %d, medsdispls ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", medsdispls[i]);
  printf("\n");

  printf("myrank: %d, medrdispls ", myrank);
  for (i=0; i<ntasks; i++)
    printf("%d ", medrdispls[i]);
  printf("\n");



  /* allocate send and recv buffers */
  buffer1 = (char *)malloc(totalscnts*sizeof(char));
  recv_buffer1 = (char *)malloc(totalrcnts*sizeof(char));

  buffer2 = (int *)malloc(totalscnts*sizeof(int)); 
  recv_buffer2 = (int *)malloc(totalrcnts*sizeof(int)); 

  buffer3 = (unsigned int *)malloc(totalscnts*sizeof(unsigned int));
  recv_buffer3 = (unsigned int *)malloc(totalrcnts*sizeof(unsigned int));

  buffer4 = (double *)malloc(totalscnts*sizeof(double));
  recv_buffer4 = (double *)malloc(totalrcnts*sizeof(double));

  medbuffer1 = (char *)malloc(totalmedscnts*sizeof(char));
  recv_medbuffer1 = (char *)malloc(totalmedrcnts*sizeof(char));

  medbuffer2 = (int *)malloc(totalmedscnts*sizeof(int));
  recv_medbuffer2 = (int *)malloc(totalmedrcnts*sizeof(int));

  medbuffer3 = (unsigned int *)malloc(totalmedscnts*sizeof(unsigned int));
  recv_medbuffer3 = (unsigned int *)malloc(totalmedrcnts*sizeof(unsigned int));

  medbuffer4 = (double *)malloc(totalmedscnts*sizeof(double));
  recv_medbuffer4 = (double *)malloc(totalmedrcnts*sizeof(double)); 

  lrgbuffer1 = (char *)malloc(totallrgscnts*sizeof(char));
  recv_lrgbuffer1 = (char *)malloc(totallrgrcnts*sizeof(char));

  lrgbuffer2 = (int *)malloc(totallrgscnts*sizeof(int));
  recv_lrgbuffer2 = (int *)malloc(totallrgrcnts*sizeof(int));

  lrgbuffer3 = (unsigned int *)malloc(totallrgscnts*sizeof(unsigned int));
  recv_lrgbuffer3 = (unsigned int *)malloc(totallrgrcnts*sizeof(unsigned int));

  lrgbuffer4 = (double *)malloc(totallrgscnts*sizeof(double));
  recv_lrgbuffer4 = (double *)malloc(totallrgrcnts*sizeof(double));


  /* initialize all send buffers */
  init_buffer(buffer1, totalscnts, TCHAR);
  init_buffer(buffer2, totalscnts, TINT);
  init_buffer(buffer3, totalscnts, TUINT);
  init_buffer(buffer4, totalscnts, TDOUBLE);

  init_buffer(medbuffer1, totalmedscnts, TCHAR);
  init_buffer(medbuffer2, totalmedscnts, TINT);
  init_buffer(medbuffer3, totalmedscnts, TUINT);
  init_buffer(medbuffer4, totalmedscnts, TDOUBLE);
 
  init_buffer(lrgbuffer1, totallrgscnts, TCHAR);
  init_buffer(lrgbuffer2, totallrgscnts, TINT);
  init_buffer(lrgbuffer3, totallrgscnts, TUINT);
  init_buffer(lrgbuffer4, totallrgscnts, TDOUBLE);

  printf("myrank: %d, buffer1: ", myrank);
  for (i=0; i<totalscnts; i++)
    printf("%c ", buffer1[i]);
  printf("\n");
  
  /*
  MPI_Alltoallv(buffer1, scounts, sdispls, MPI_CHAR, recv_buffer1,
              rcounts, rdispls, MPI_CHAR, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++) 
    if (memcmp(buffer1+sdispls[myrank], recv_buffer1+rdispls[i],
						      rcounts[myrank]) != 0)  
      printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Alltoallv(buffer2, scounts, sdispls, MPI_INT, recv_buffer2, rcounts,
		rdispls, MPI_INT, MPI_COMM_WORLD); 
  for (i=0; i<ntasks; i++)
    if (memcmp(buffer2+sdispls[myrank], recv_buffer2+rdispls[i],
	       rcounts[myrank]) != 0)
      printf("rank: %d, buffer2 received incorrectly\n", myrank);

  MPI_Alltoallv(buffer3, scounts, sdispls, MPI_UNSIGNED, recv_buffer3,
		rcounts, rdispls, MPI_UNSIGNED, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++)
    if (memcmp(buffer3+sdispls[myrank], recv_buffer3+rdispls[i],
	       rcounts[myrank]) != 0)
      printf("rank: %d, buffer3 received incorrectly\n", myrank);

  MPI_Alltoallv(buffer4, scounts, sdispls, MPI_DOUBLE, recv_buffer4,
		rcounts, rdispls, MPI_DOUBLE, MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++)
    if (memcmp(buffer4+sdispls[myrank], recv_buffer4+rdispls[i],
	       rcounts[myrank]) != 0)
      printf("rank: %d, buffer4 received incorrectly\n", myrank);

  MPI_Alltoallv(medbuffer1, medscounts, medsdispls, MPI_CHAR,
		recv_medbuffer1, medrcounts, medrdispls, MPI_CHAR,
		MPI_COMM_WORLD);   
  for (i=0; i<ntasks; i++)
    if (memcmp(medbuffer1+medsdispls[myrank], recv_medbuffer1+medrdispls[i],
	       medrcounts[myrank]) != 0) 
      printf("rank: %d, medbuffer1 received incorrectly at i: %d\n",
	     myrank, i); 

  MPI_Alltoallv(medbuffer2, medscounts, medsdispls, MPI_INT,
                recv_medbuffer2, medrcounts, medrdispls, MPI_INT,
                MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++)
    if (memcmp(medbuffer2+medsdispls[myrank],
	       recv_medbuffer2+medrdispls[i],
               medrcounts[myrank]) != 0)
      printf("rank: %d, medbuffer2 received incorrectly at i: %d\n",
             myrank, i);

  MPI_Alltoallv(medbuffer3, medscounts, medsdispls, MPI_UNSIGNED,
                recv_medbuffer3, medrcounts, medrdispls, MPI_UNSIGNED,
                MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++)
    if (memcmp(medbuffer3+medsdispls[myrank],
                recv_medbuffer3+medrdispls[i],
               medrcounts[myrank]) != 0)
      printf("rank: %d, medbuffer3 received incorrectly at i: %d\n",
             myrank, i);

  MPI_Alltoallv(medbuffer4, medscounts, medsdispls, MPI_DOUBLE,
                recv_medbuffer4, medrcounts, medrdispls, MPI_DOUBLE,
                MPI_COMM_WORLD);
  for (i=0; i<ntasks; i++)
    if (memcmp(medbuffer4+medsdispls[myrank],
                recv_medbuffer4+medrdispls[i],
               medrcounts[myrank]) != 0)
      printf("rank: %d, medbuffer4 received incorrectly at i: %d\n",
             myrank, i);
*/
  MPI_Alltoallv(lrgbuffer1, lrgscounts, lrgsdispls, MPI_CHAR,
		recv_lrgbuffer1, lrgrcounts, lrgrdispls, MPI_CHAR,
		MPI_COMM_WORLD);   
  for (i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer1+lrgsdispls[myrank], recv_lrgbuffer1+lrgrdispls[i],
	       lrgrcounts[myrank]) != 0)
      {
	printf("rank: %d, lrgbuffer1 received incorrectly at i: %d\n",
	       myrank, i);
      }

  MPI_Alltoallv(lrgbuffer2, lrgscounts, lrgsdispls, MPI_INT,
		recv_lrgbuffer2, lrgrcounts, lrgrdispls, MPI_INT,
		MPI_COMM_WORLD);   
  for (i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer2+lrgsdispls[myrank], recv_lrgbuffer2+lrgrdispls[i],
	       lrgrcounts[myrank]) != 0)
      {
	printf("rank: %d, lrgbuffer2 received incorrectly at i: %d\n",
	       myrank, i);
      }

  MPI_Alltoallv(lrgbuffer3, lrgscounts, lrgsdispls, MPI_UNSIGNED,
		recv_lrgbuffer3, lrgrcounts, lrgrdispls, MPI_UNSIGNED,
		MPI_COMM_WORLD);   
  for (i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer3+lrgsdispls[myrank], recv_lrgbuffer3+lrgrdispls[i],
	       lrgrcounts[myrank]) != 0)
      {
	printf("rank: %d, lrgbuffer3 received incorrectly at i: %d\n",
	       myrank, i);
      }

  MPI_Alltoallv(lrgbuffer4, lrgscounts, lrgsdispls, MPI_DOUBLE,
		recv_lrgbuffer4, lrgrcounts, lrgrdispls, MPI_DOUBLE,
		MPI_COMM_WORLD);   
  for (i=0; i<ntasks; i++)
    if (memcmp(lrgbuffer4+lrgsdispls[myrank], recv_lrgbuffer4+lrgrdispls[i],
	       lrgrcounts[myrank]) != 0)
      {
	printf("rank: %d, lrgbuffer4 received incorrectly at i: %d\n",
	       myrank, i);
      }

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}
