#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include "mpi.h"

/* general functionality test for MPI_Bcast */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
static void init_buffer(void *buf, int count, int type);

int main(
    int argc,
    char *argv[])
{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */
  struct timeval starttime;
  struct timeval endtime;
  clock_t start, end;
  unsigned long totalclocktime;
  double totalcputime;
  int totalticks;
  int ntasks;

  /* test datatypes: char, int, unsigned int and double */
  /* one of each datatype */
  char buffer1;
  char dup_buffer1;
  int buffer2;
  int dup_buffer2;
  unsigned int buffer3;
  unsigned int dup_buffer3;
  double buffer4;
  double dup_buffer4;

  /* a few of each datatype */
  char *medbuffer1;
  char *dup_medbuffer1;
  int *medbuffer2;
  int *dup_medbuffer2;
  unsigned int *medbuffer3;
  unsigned int *dup_medbuffer3;
  double *medbuffer4;
  double *dup_medbuffer4;
  
  /* larger number of each datatype */
  char *lrgbuffer1;
  char *dup_lrgbuffer1;
  int *lrgbuffer2;
  int *dup_lrgbuffer2;
  unsigned int *lrgbuffer3;
  unsigned int *dup_lrgbuffer3;
  double *lrgbuffer4;
  double *dup_lrgbuffer4;

  /* test on these diff counts */
  int count = 1;
  int medcount = 1024;
  int lrgcount = 25600;

  /* allocate buffers */
  medbuffer1 = (char *)malloc(medcount*sizeof(char)); /* 1K */
  dup_medbuffer1 = (char *)malloc(medcount*sizeof(char)); /* 1K */

  medbuffer2 = (int *)malloc(medcount*sizeof(int)); /* 4K */
  dup_medbuffer2 = (int *)malloc(medcount*sizeof(int)); /* 4K */

  medbuffer3 = (unsigned int *)malloc(medcount*sizeof(unsigned int)); /* 4K */
  dup_medbuffer3 = (unsigned int *)malloc(medcount*sizeof(unsigned int)); /* 4K*/

  medbuffer4 = (double *)malloc(medcount*sizeof(double)); /* 8K */
  dup_medbuffer4 = (double *)malloc(medcount*sizeof(double)); /* 8K */

  lrgbuffer1 = (char *)malloc(lrgcount*sizeof(char)); /*100K*/
  dup_lrgbuffer1 = (char *)malloc(lrgcount*sizeof(char)); /*100K*/

  lrgbuffer2 = (int *)malloc(lrgcount*sizeof(int)); /* 100K */
  dup_lrgbuffer2 = (int *)malloc(lrgcount*sizeof(int)); /* 100K */

  lrgbuffer3 = (unsigned int *)malloc(lrgcount*sizeof(unsigned int)); /* 100K */
  dup_lrgbuffer3 = (unsigned int *)malloc(lrgcount*sizeof(unsigned int)); /* 100K */

  lrgbuffer4 = (double *)malloc(lrgcount*sizeof(double)); /*200K */
  dup_lrgbuffer4 = (double *)malloc(lrgcount*sizeof(double)); /*200K */

  /* initialize root's buffers for broadcasting */
  if ( myrank == 0 )
    {
      init_buffer(&buffer1, count, TCHAR);
      init_buffer(&buffer2, count, TINT);
      init_buffer(&buffer3, count, TUINT);
      init_buffer(&buffer4, count, TDOUBLE);

      init_buffer(medbuffer1, medcount, TCHAR);
      init_buffer(medbuffer2, medcount, TINT);
      init_buffer(medbuffer3, medcount, TUINT);
      init_buffer(medbuffer4, medcount, TDOUBLE);

      init_buffer(lrgbuffer1, lrgcount, TCHAR);
      init_buffer(lrgbuffer2, lrgcount, TINT);
      init_buffer(lrgbuffer3, lrgcount, TUINT);
      init_buffer(lrgbuffer4, lrgcount, TDOUBLE);
    }

  /* initialize the duplicate buffers with same content as buffers
     above for confirmation later that processes received match these
     buffers */    
  init_buffer(&dup_buffer1, count, TCHAR);
  init_buffer(&dup_buffer2, count, TINT);
  init_buffer(&dup_buffer3, count, TUINT);
  init_buffer(&dup_buffer4, count, TDOUBLE);
  
  init_buffer(dup_medbuffer1, medcount, TCHAR);
  init_buffer(dup_medbuffer2, medcount, TINT);
  init_buffer(dup_medbuffer3, medcount, TUINT);
  init_buffer(dup_medbuffer4, medcount, TDOUBLE);

  init_buffer(dup_lrgbuffer1, lrgcount, TCHAR);
  init_buffer(dup_lrgbuffer2, lrgcount, TINT);
  init_buffer(dup_lrgbuffer3, lrgcount, TUINT);
  init_buffer(dup_lrgbuffer4, lrgcount, TDOUBLE);
    
  MPI_Comm_size(MPI_COMM_WORLD, &ntasks);  		      
	
  printf("ntasks: %d\n", ntasks);

  /*  gettimeofday(&starttime, NULL); */
  start = clock();
  MPI_Bcast(&buffer1, count, MPI_CHAR, 0, MPI_COMM_WORLD);
  end = clock();
  /*   gettimeofday(&endtime, NULL); */
  totalcputime = ((double) (end - start)) / CLOCKS_PER_SEC;
  printf("cputime: %f\n", totalcputime);

  if (memcmp(&buffer1, &dup_buffer1, count) != 0)
    printf("rank: %d, buffer1 received incorrectly\n", myrank);

  MPI_Bcast(&buffer2, count, MPI_INT, 0, MPI_COMM_WORLD);
  if (memcmp(&buffer2, &dup_buffer2, count) != 0)
    printf("rank: %d, buffer2 received incorrectly\n", myrank);

  MPI_Bcast(&buffer3, count, MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  if (memcmp(&buffer3, &dup_buffer3, count) != 0)
    printf("rank: %d, buffer3 received incorrectly\n", myrank);
  
  MPI_Bcast(&buffer4, count, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  if (memcmp(&buffer4, &dup_buffer4, count) != 0)
    printf("rank: %d, buffer4 received incorrectly\n", myrank);

  MPI_Bcast(medbuffer1, medcount, MPI_CHAR, 0, MPI_COMM_WORLD);
  if (memcmp(medbuffer1, dup_medbuffer1, medcount) != 0)
    printf("rank: %d, medbuffer1 received incorrectly\n", myrank);

  MPI_Bcast(medbuffer2, medcount, MPI_INT, 0, MPI_COMM_WORLD);
  if (memcmp(medbuffer2, dup_medbuffer2, medcount) != 0)
    printf("rank: %d, medbuffer2 received incorrectly\n", myrank);

  MPI_Bcast(medbuffer3, medcount, MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  if (memcmp(medbuffer3, dup_medbuffer3, medcount) != 0)
    printf("rank: %d, medbuffer3 received incorrectly\n", myrank);

  MPI_Bcast(medbuffer4, medcount, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  if (memcmp(medbuffer4, dup_medbuffer4, medcount) != 0)
    printf("rank: %d, medbuffer4 received incorrectly\n", myrank);

  MPI_Bcast(lrgbuffer1, lrgcount, MPI_CHAR, 0, MPI_COMM_WORLD);
    if (memcmp(lrgbuffer1, dup_lrgbuffer1, lrgcount) != 0)
    printf("rank: %d, lrgbuffer1 received incorrectly\n", myrank);

  MPI_Bcast(lrgbuffer2, lrgcount, MPI_INT, 0, MPI_COMM_WORLD);
    if (memcmp(lrgbuffer2, dup_lrgbuffer2, lrgcount) != 0)
    printf("rank: %d, lrgbuffer2 received incorrectly\n", myrank);

  MPI_Bcast(lrgbuffer3, lrgcount, MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  if (memcmp(lrgbuffer3, dup_lrgbuffer3, lrgcount) != 0)
    printf("rank: %d, lrgbuffer3 received incorrectly\n", myrank);
  int clock_ticks = sysconf(_SC_CLK_TCK);
  start = clock() * clock_ticks;
  MPI_Bcast(lrgbuffer4, lrgcount, MPI_DOUBLE, 0, MPI_COMM_WORLD);
  end = clock() * clock_ticks;
  totalticks = end - start;
  printf("# of ticks for lrgbuffer: %d\n", totalticks);
  if (memcmp(lrgbuffer4, dup_lrgbuffer4, lrgcount) != 0)
    printf("rank: %d, lrgbuffer4 received incorrectly\n", myrank);

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

static void init_buffer(void *buf, int count, int type)
{
  int i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}
