#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include "mpi.h"

/* used to time both sequential and tree versions of MPI_Bcast on the quad machine*/

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);
static void barrier(void);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  struct timeval starttime;
  struct timeval endtime; 
  clock_t start, end; 

  int ntasks;
  long i;

  /* just test on one datatype to be consistent for timing */
  int *buffer1;
  int *buffer2;
  int *buffer3;
  int *buffer4;
  int *buffer5;

  int *dupbuffer1;
  int *dupbuffer2;
  int *dupbuffer3;
  int *dupbuffer4;
  int *dupbuffer5;

  int count1 = 51200;
  int count2 = 102400; 
  int count3 = 204800;
  int count4 = 409600;
  int count5 = 819200;

  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */

  MPI_Comm_size(MPI_COMM_WORLD, &ntasks);  		      

  /* allocate buffers */
  buffer1 = (int *)malloc(count1*sizeof(int));
  buffer2 = (int *)malloc(count2*sizeof(int));
  buffer3 = (int *)malloc(count3*sizeof(int));
  buffer4 = (int *)malloc(count4*sizeof(int));
  buffer5 = (int *)malloc(count5*sizeof(int));
  dupbuffer1 = (int *)malloc(count1*sizeof(int));
  dupbuffer2 = (int *)malloc(count2*sizeof(int));
  dupbuffer3 = (int *)malloc(count3*sizeof(int));
  dupbuffer4 = (int *)malloc(count4*sizeof(int));
  dupbuffer5 = (int *)malloc(count5*sizeof(int));


  /* initialize root's buffers for broadcasting */
  if ( myrank == 0 ) {
    init_buffer(buffer1, count1, TINT);
    /*for(i=0; i < count1; i++)
      printf("%d ", buffer1[i]);
    printf("\n");
    */
    init_buffer(buffer2, count2, TINT);
    init_buffer(buffer3, count3, TINT);
    init_buffer(buffer4, count4, TINT);
    init_buffer(buffer5, count5, TINT);
  }

  init_buffer(dupbuffer1, count1, TINT);
  /*  for(i=0; i < count1; i++)
    printf("%d ", dupbuffer1[i]);
  printf("\n");
  */
  init_buffer(dupbuffer2, count2, TINT);
  init_buffer(dupbuffer3, count3, TINT);
  init_buffer(dupbuffer4, count4, TINT);
  init_buffer(dupbuffer5, count5, TINT);

  /* note cpu time */
  if(myrank == 0)
    start = clock(); 

  MPI_Bcast(buffer1, count1, MPI_INT, 0, MPI_COMM_WORLD);
  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count1, ((float) (end - start)) / CLOCKS_PER_SEC);
  }
  if (memcmp(buffer1, dupbuffer1, count1) != 0) {
    printf("rank: %d, buffer1 received incorrectly\n", myrank);
    /*    for(i=0; i < count1; i++)
      printf("%d ", buffer1[i]);
    printf("\n");
    */
  }

  if(myrank == 0)
    start = clock(); 

  MPI_Bcast(buffer2, count2, MPI_INT, 0, MPI_COMM_WORLD);
  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count2, ((float) (end - start)) / CLOCKS_PER_SEC);
  }
  if (memcmp(buffer2, dupbuffer2, count2) != 0)
    printf("rank: %d, buffer2 received incorrectly\n", myrank);

  if(myrank == 0)
    start = clock(); 

  MPI_Bcast(buffer3, count3, MPI_INT, 0, MPI_COMM_WORLD);
  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count3, ((float) (end - start)) / CLOCKS_PER_SEC);
  }
  if (memcmp(buffer3, dupbuffer3, count3) != 0)
    printf("rank: %d, buffer3 received incorrectly\n", myrank);

  if(myrank == 0)
    start = clock(); 

  MPI_Bcast(buffer4, count4, MPI_INT, 0, MPI_COMM_WORLD);
  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count4, ((float) (end - start)) / CLOCKS_PER_SEC);
  }
  if (memcmp(buffer4, dupbuffer4, count4) != 0)
    printf("rank: %d, buffer4 received incorrectly\n", myrank);


  if(myrank == 0)
    start = clock(); 

  MPI_Bcast(buffer5, count5, MPI_INT, 0, MPI_COMM_WORLD);
  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count5, ((float) (end - start)) / CLOCKS_PER_SEC);
  }
  if (memcmp(buffer5, dupbuffer5, count5) != 0)
    printf("rank: %d, buffer5 received incorrectly\n", myrank);

  /* note wall clock time */
  if(myrank == 0)
    gettimeofday(&starttime, NULL);
  
  MPI_Bcast(buffer1, count1, MPI_INT, 0, MPI_COMM_WORLD);
  barrier();

  if(myrank == 0) {
    gettimeofday(&endtime, NULL);
    if (starttime.tv_usec > endtime.tv_usec) {
      endtime.tv_usec += 1000000;
      endtime.tv_sec--;
    }
    printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
	   count1, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
							 starttime.tv_usec));
  }
  if (memcmp(buffer1, dupbuffer1, count1) != 0)
    printf("rank: %d, buffer1 received incorrectly\n", myrank);
  
  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  MPI_Bcast(buffer2, count2, MPI_INT, 0, MPI_COMM_WORLD);
  barrier();
  
  if(myrank == 0) {
    gettimeofday(&endtime, NULL);
    if (starttime.tv_usec > endtime.tv_usec) {
      endtime.tv_usec += 1000000;
      endtime.tv_sec--;
    }
    printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
           count2, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
							 starttime.tv_usec));
  }
  if (memcmp(buffer2, dupbuffer2, count2) != 0)
    printf("rank: %d, buffer2 received incorrectly\n", myrank);

  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  MPI_Bcast(buffer3, count3, MPI_INT, 0, MPI_COMM_WORLD);
  barrier();

  if(myrank == 0) {
    gettimeofday(&endtime, NULL);
    if (starttime.tv_usec > endtime.tv_usec) {
      endtime.tv_usec += 1000000;
      endtime.tv_sec--;
    }
    printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
           count3, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
							 starttime.tv_usec));
  }
  if (memcmp(buffer3, dupbuffer3, count3) != 0)
    printf("rank: %d, buffer3 received incorrectly\n", myrank);

  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  MPI_Bcast(buffer4, count4, MPI_INT, 0, MPI_COMM_WORLD);
  barrier();

  if(myrank == 0) {
    gettimeofday(&endtime, NULL);
    if (starttime.tv_usec > endtime.tv_usec) {
      endtime.tv_usec += 1000000;
      endtime.tv_sec--;
    }
    printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
           count4, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
                                                 starttime.tv_usec));
  }
  if (memcmp(buffer4, dupbuffer4, count4) != 0)
    printf("rank: %d, buffer4 received incorrectly\n", myrank);

  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  MPI_Bcast(buffer5, count5, MPI_INT, 0, MPI_COMM_WORLD);
  barrier();

  if(myrank == 0) {
    gettimeofday(&endtime, NULL);
    if (starttime.tv_usec > endtime.tv_usec) {
      endtime.tv_usec += 1000000;
      endtime.tv_sec--;
    }
    printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
           count5, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
                                                 starttime.tv_usec));
  }
  if (memcmp(buffer5, dupbuffer5, count5) != 0)
    printf("rank: %d, buffer5 received incorrectly\n", myrank);

    MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

void init_buffer(void *buf, int count, int type)
{
  int i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=(int)i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}

static void barrier(void)
{
  int             maxrank, myrank, rank;
  int             buf;
  int             msg = 1;
  MPI_Status      status;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&maxrank);        /* #processes in application = N */
  MPI_Comm_rank(MPI_COMM_WORLD,   /* always use this */
		&myrank);         /* process rank, 0 thru N-1 */

  if(myrank == 0)
    {
      for ( rank = 0; rank < maxrank; rank++ ) {
	if ( rank != myrank)
	  MPI_Recv(&buf, 1, MPI_INT, rank, 0, MPI_COMM_WORLD, &status);
      }
    }
  else
    MPI_Send(&msg, 1, MPI_INT, 0, 0, MPI_COMM_WORLD);
}
