#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include "mpi.h"

/* used to time both sequential and tree versions of MPI_Bcast on the home
   local machines */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);
static void barrier(void);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */
  struct timeval starttime;
  struct timeval endtime; 
  clock_t start, end; 

  int ntasks;
  long i;
  int temp;

  /* just test on one datatype to be consistent for timing */
  int *buffer;
  int *medbuffer;
  int *lrgbuffer;

  int count = 51200;
  int medcount = 102400;
  int lrgcount = 204800;

  MPI_Comm_size(MPI_COMM_WORLD, &ntasks);  		      

  /* allocate buffers */
  buffer = (int *)malloc(count*sizeof(int));
  medbuffer = (int *)malloc(medcount*sizeof(int));
  lrgbuffer = (int *)malloc(lrgcount*sizeof(int));

  /* initialize root's buffers for broadcasting */
  if ( myrank == 0 )
    {
      init_buffer(buffer, count, TINT);
      init_buffer(medbuffer, medcount, TINT);
      init_buffer(lrgbuffer, lrgcount, TINT);
    }


  /* note cpu time */
  if(myrank == 0)
    start = clock(); 

  for(i=0; i < 3; i++)  
    MPI_Bcast(buffer, count, MPI_INT, 0, MPI_COMM_WORLD);

  barrier(); 
  
  if(myrank == 0) {
    end = clock(); 
    printf("total cputime for count %d: %f\n", count, ((float) (end -
								 start)) /
	   CLOCKS_PER_SEC);
  }

  if(myrank == 0)
    start = clock();

  for(i=0; i < 3; i++)
    MPI_Bcast(medbuffer, medcount, MPI_INT, 0, MPI_COMM_WORLD);

  barrier();

  if(myrank == 0) {
    end = clock();
    printf("total cputime for count %d: %f\n", medcount, ((float) (end -
								start)) /
           CLOCKS_PER_SEC);
  }

  if(myrank == 0)
    start = clock();

  for(i=0; i < 3; i++)
    MPI_Bcast(lrgbuffer, lrgcount, MPI_INT, 0, MPI_COMM_WORLD);

  barrier();

  if(myrank == 0) {
    end = clock();
    printf("total cputime for count %d: %f\n", lrgcount, ((float) (end -
								start)) /
           CLOCKS_PER_SEC);
  }



  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  for(i=0; i < 3; i++)
    MPI_Bcast(buffer, count, MPI_INT, 0, MPI_COMM_WORLD);

  barrier();

  if(myrank == 0)
    gettimeofday(&endtime, NULL);

  if(myrank == 0)
    {
      if (starttime.tv_usec > endtime.tv_usec) {
        endtime.tv_usec += 1000000;
        endtime.tv_sec--;
      }
      printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n",
             count, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec -
                                                   starttime.tv_usec));
    }

  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  for(i=0; i < 3; i++)
    MPI_Bcast(medbuffer, medcount, MPI_INT, 0, MPI_COMM_WORLD);

  barrier();

  if(myrank == 0)
    gettimeofday(&endtime, NULL);

  if(myrank == 0)
    {
      if (starttime.tv_usec > endtime.tv_usec) {
        endtime.tv_usec += 1000000;
        endtime.tv_sec--;
      }
      printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n", medcount, (endtime.tv_sec - starttime.tv_sec),
	     (endtime.tv_usec - starttime.tv_usec));
    }


  if(myrank == 0)
    gettimeofday(&starttime, NULL);

  for(i=0; i < 3; i++)
    MPI_Bcast(lrgbuffer, lrgcount, MPI_INT, 0, MPI_COMM_WORLD);

  barrier();

  if(myrank == 0)
    gettimeofday(&endtime, NULL);

  if(myrank == 0)
    {
      if (starttime.tv_usec > endtime.tv_usec) {
        endtime.tv_usec += 1000000;
        endtime.tv_sec--;
      }
      printf("total wallclock time for count %d:  %d seconds and %lu microseconds\n", lrgcount, (endtime.tv_sec - starttime.tv_sec), (endtime.tv_usec - starttime.tv_usec)); 
    }

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

void init_buffer(void *buf, int count, int type)
{
  int i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}

static void barrier(void)
{
  int             maxrank, myrank, rank;
  int             buf;
  int             msg = 1;
  MPI_Status      status;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&maxrank);        /* #processes in application = N */
  MPI_Comm_rank(MPI_COMM_WORLD,   /* always use this */
		&myrank);         /* process rank, 0 thru N-1 */

  if(myrank == 0)
    {
      for ( rank = 0; rank < maxrank; rank++ ) {
	if ( rank != myrank)
	  MPI_Recv(&buf, 1, MPI_INT, rank, 0, MPI_COMM_WORLD, &status);
      }
    }
  else
    MPI_Send(&msg, 1, MPI_INT, 0, 0, MPI_COMM_WORLD);
}
