#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* general functionality test for MPI_Gather */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  
  MPI_Init(&argc, &argv);	/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	/* always use this */
		&myrank);	/* process rank, 0 thru N-1 */

  int ntasks;
  int i;
  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
                &ntasks);

  /* test datatypes: char, int, unsigned int and double */

  char *buffer1;
  char *recv_buffer1;
  int *buffer2;
  int *recv_buffer2;
  unsigned int *buffer3;
  unsigned int *recv_buffer3;
  double *buffer4;
  double *recv_buffer4;

  /* a few of each datatype */
  char *medbuffer1;
  char *recv_medbuffer1;
  int *medbuffer2;
  int *recv_medbuffer2;
  unsigned int *medbuffer3;
  unsigned int *recv_medbuffer3;
  double *medbuffer4;
  double *recv_medbuffer4;

  /* larger number of each datatype */
  char *lrgbuffer1;
  char *recv_lrgbuffer1;
  int *lrgbuffer2;
  int *recv_lrgbuffer2;
  unsigned int *lrgbuffer3;
  unsigned int *recv_lrgbuffer3;
  double *lrgbuffer4;
  double *recv_lrgbuffer4;

  int count = 1;
  int medcount = 1024;
  int lrgcount = 25600;

  /* allocate buffers */
  buffer1 = (char *)malloc(count*sizeof(char));
  recv_buffer1 = (char *)malloc(count*ntasks*sizeof(char));

  buffer2 = (int *)malloc(count*sizeof(int)); 
  recv_buffer2 = (int *)malloc(count*ntasks*sizeof(int)); 

  buffer3 = (unsigned int *)malloc(count*sizeof(unsigned int));
  recv_buffer3 = (unsigned int *)malloc(count*ntasks*sizeof(unsigned int));

  buffer4 = (double *)malloc(count*sizeof(double));
  recv_buffer4 = (double *)malloc(count*ntasks*sizeof(double));

  medbuffer1 = (char *)malloc(medcount*sizeof(char));
  recv_medbuffer1 = (char *)malloc(medcount*ntasks*sizeof(char));

  medbuffer2 = (int *)malloc(medcount*sizeof(int));
  recv_medbuffer2 = (int *)malloc(medcount*ntasks*sizeof(int));

  medbuffer3 = (unsigned int *)malloc(medcount*sizeof(unsigned int));
  recv_medbuffer3 = (unsigned int *)malloc(medcount*ntasks*sizeof(unsigned int));

  medbuffer4 = (double *)malloc(medcount*sizeof(double));
  recv_medbuffer4 = (double *)malloc(medcount*ntasks*sizeof(double)); 

  lrgbuffer1 = (char *)malloc(lrgcount*sizeof(char));
  recv_lrgbuffer1 = (char *)malloc(lrgcount*ntasks*sizeof(char));

  lrgbuffer2 = (int *)malloc(lrgcount*sizeof(int));
  recv_lrgbuffer2 = (int *)malloc(lrgcount*ntasks*sizeof(int));

  lrgbuffer3 = (unsigned int *)malloc(lrgcount*sizeof(unsigned int));
  recv_lrgbuffer3 = (unsigned int *)malloc(lrgcount*ntasks*sizeof(unsigned int));

  lrgbuffer4 = (double *)malloc(lrgcount*sizeof(double));
  recv_lrgbuffer4 = (double *)malloc(lrgcount*ntasks*sizeof(double));


  /* initialize send buffers */
  init_buffer(buffer1, count, TCHAR);
  init_buffer(buffer2, count, TINT);
  init_buffer(buffer3, count, TUINT);
  init_buffer(buffer4, count, TDOUBLE);

  init_buffer(medbuffer1, medcount, TCHAR);
  init_buffer(medbuffer2, medcount, TINT);
  init_buffer(medbuffer3, medcount, TUINT);
  init_buffer(medbuffer4, medcount, TDOUBLE);
    
  init_buffer(lrgbuffer1, lrgcount, TCHAR);
  init_buffer(lrgbuffer2, lrgcount, TINT);
  init_buffer(lrgbuffer3, lrgcount, TUINT);
  init_buffer(lrgbuffer4, lrgcount, TDOUBLE);

  MPI_Gather(buffer1, count, MPI_CHAR, recv_buffer1, count, MPI_CHAR, 0,
	      MPI_COMM_WORLD);  
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(buffer1, recv_buffer1+i, count) != 0)
	  printf("rank: %d, buffer1 received incorrectly\n", myrank);
    }

  MPI_Gather(buffer2, count, MPI_INT, recv_buffer2, count, MPI_INT, 0,
	     MPI_COMM_WORLD);  
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(buffer2, recv_buffer2+i, count) != 0)
	  printf("rank: %d, buffer2 received incorrectly\n", myrank);
    }

  MPI_Gather(buffer3, count, MPI_UNSIGNED, recv_buffer3, count, MPI_UNSIGNED, 0, MPI_COMM_WORLD); 
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)  
	if (memcmp(buffer3, recv_buffer3+i, count) != 0)
	  printf("rank: %d, buffer3 received incorrectly\n", myrank);
    }

  MPI_Gather(buffer4, count, MPI_DOUBLE, recv_buffer4, count, MPI_DOUBLE, 0,
	      MPI_COMM_WORLD);  
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(buffer4, recv_buffer4+i, count) != 0)
	  printf("rank: %d, buffer4 received incorrectly\n", myrank);
    }

  MPI_Gather(medbuffer1, medcount, MPI_CHAR, recv_medbuffer1, medcount, MPI_CHAR,
	      0, MPI_COMM_WORLD); 
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(medbuffer1, recv_medbuffer1+(i*medcount), medcount) != 0)
	  printf("rank: %d, medbuffer1 received incorrectly\n", myrank);
    }

  MPI_Gather(medbuffer2, medcount, MPI_INT, recv_medbuffer2, medcount, MPI_INT,
	      0, MPI_COMM_WORLD); 
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(medbuffer2, recv_medbuffer2+(i*medcount), medcount) != 0)
	  printf("rank: %d, medbuffer2 received incorrectly\n", myrank);
    }

  MPI_Gather(medbuffer3, medcount, MPI_UNSIGNED, recv_medbuffer3, medcount,
	      MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(medbuffer3, recv_medbuffer3+(i*medcount), medcount) != 0)
	  printf("rank: %d, medbuffer3 received incorrectly\n", myrank);
    }

  MPI_Gather(medbuffer4, medcount, MPI_DOUBLE, recv_medbuffer4, medcount,
	      MPI_DOUBLE, 0, MPI_COMM_WORLD); 
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(medbuffer4, recv_medbuffer4+(i*medcount), medcount) != 0)
	  printf("rank: %d, medbuffer4 received incorrectly\n", myrank);
    }

  MPI_Gather(lrgbuffer1, lrgcount, MPI_CHAR, recv_lrgbuffer1, lrgcount,
	      MPI_CHAR, 0, MPI_COMM_WORLD);
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(lrgbuffer1, recv_lrgbuffer1+(i*lrgcount), lrgcount) != 0)
	  printf("rank: %d, lrgbuffer1 received incorrectly\n", myrank);
    }

  MPI_Gather(lrgbuffer2, lrgcount, MPI_INT, recv_lrgbuffer2, lrgcount,
	      MPI_INT, 0, MPI_COMM_WORLD);
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(lrgbuffer2, recv_lrgbuffer2+(i*lrgcount), lrgcount) != 0)
	  printf("rank: %d, lrgbuffer2 received incorrectly\n", myrank);
    }

  MPI_Gather(lrgbuffer3, lrgcount, MPI_UNSIGNED, recv_lrgbuffer3, lrgcount,
              MPI_UNSIGNED, 0, MPI_COMM_WORLD);
  if (myrank == 0)
    {
     for(i=0; i<ntasks; i++)
	if (memcmp(lrgbuffer3, recv_lrgbuffer3+(i*lrgcount), lrgcount) != 0)
	  printf("rank: %d, lrgbuffer3 received incorrectly\n", myrank);
    }

  MPI_Gather(lrgbuffer4, lrgcount, MPI_DOUBLE, recv_lrgbuffer4, lrgcount,
	     MPI_DOUBLE, 0, MPI_COMM_WORLD);
  if (myrank == 0)
    {
      for(i=0; i<ntasks; i++)
	if (memcmp(lrgbuffer4, recv_lrgbuffer4+(i*lrgcount), lrgcount) != 0)
	  printf("rank: %d, lrgbuffer4 received incorrectly\n", myrank);
    }

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
	((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
	((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
	((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
	((double*)buf)[i]=(double)i+1;
      break;
    }
}
