#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* test for MPI_Reduce on MPI_MAXLOC operation */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  struct { 
    int val; 
    int rank; 
  } isendbuf[ntasks], irecvbuf[ntasks], is[1], ir[1]; 

  struct { 
    float val; 
    int rank; 
  } fsendbuf[ntasks], frecvbuf[ntasks], fs[1], fr[1];

    
  /* initialize send buffers */
  is[0].val = myrank;
  is[0].rank = myrank;
  printf("myrank: %d send int array vals: ", myrank);
  for(i=0; i<ntasks; i++) { 
      isendbuf[i].val = i+myrank;
      printf("%d ", isendbuf[i].val);
      isendbuf[i].rank = myrank; 
    }
  printf("\n");

  fs[0].val = (float)myrank;
  fs[0].rank = myrank;
  printf("myrank: %d send float array vals: ", myrank);
  for(i=0; i<ntasks; i++) { 
      fsendbuf[i].val = (float)i+myrank;
      printf("%f ", fsendbuf[i].val);
      fsendbuf[i].rank = myrank; 
    }
  printf("\n");

  /* test datatype pair (int, int) */
  /* test on one-element sendbuf */
  MPI_Reduce(is, ir, 1, MPI_INT_INT, MPI_MAXLOC, 0, MPI_COMM_WORLD);

  /* test on many-element sendbuf */
  MPI_Reduce(isendbuf, irecvbuf, ntasks, MPI_INT_INT, MPI_MAXLOC, 0,
	     MPI_COMM_WORLD);
  
  /* test datatype pair (float, int) */
  /* test on one-element sendbuf */
  MPI_Reduce(fs, fr, 1, MPI_FLOAT_INT, MPI_MAXLOC, 0, MPI_COMM_WORLD);
  
  /* test on many-element sendbuf */
  MPI_Reduce(fsendbuf, frecvbuf, ntasks, MPI_FLOAT_INT, MPI_MAXLOC, 0,
	     MPI_COMM_WORLD);

  if (myrank == 0) 
    {      
      printf("recv int array:\n");
      for (i=0; i<ntasks; i++) 
	printf("i:%d val:%d index:%d\n", i, irecvbuf[i].val, irecvbuf[i].rank);

      printf("recv float array:\n");
      for (i=0; i<ntasks; i++) 
	printf("i:%d val:%f index:%d\n", i, frecvbuf[i].val, frecvbuf[i].rank);
    } 

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;
    }
}
