#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* test for MPI_Reduce on MPI_MINLOC operation */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  
  struct { 
    short val; 
    int rank; 
  } ssendbuf[ntasks], srecvbuf[ntasks], ss[1], sr[1];


  struct { 
    long double val; 
    int rank; 
  } ldsendbuf[ntasks], ldrecvbuf[ntasks], lds[1], ldr[1];

  
  /* initialize send buffers */
  ss[0].val = (short)myrank;
  ss[0].rank = myrank;
  printf("myrank: %d send short array vals: ", myrank);
  for(i=0; i<ntasks; i++) { 
      ssendbuf[i].val = (short)i+myrank;
      printf("%d ", ssendbuf[i].val);
      ssendbuf[i].rank = myrank; 
    }
  printf("\n");

  lds[0].val = (long double)myrank;
  lds[0].rank = myrank;
  printf("myrank: %d send long double array vals: ", myrank);
  for(i=0; i<ntasks; i++) { 
      ldsendbuf[i].val = (long double)i+myrank;
      printf("%Lf ", ldsendbuf[i].val);
      ldsendbuf[i].rank = myrank; 
    }
  printf("\n");


  /* test datatype pair (short, int) */
  /* test on one-element sendbuf */
  MPI_Reduce(ss, sr, 1, MPI_SHORT_INT, MPI_MINLOC, 0, MPI_COMM_WORLD);

  /* test on many-element sendbuf */
  MPI_Reduce(ssendbuf, srecvbuf, ntasks, MPI_SHORT_INT, MPI_MINLOC, 0,
	     MPI_COMM_WORLD);

  /* test datatype pair (long double, int) */
  /* test on one-element sendbuf */
  MPI_Reduce(lds, ldr, 1, MPI_LONG_DOUBLE_INT, MPI_MINLOC, 0, MPI_COMM_WORLD);
  
  /* test on many-element sendbuf */
  MPI_Reduce(ldsendbuf, ldrecvbuf, ntasks, MPI_LONG_DOUBLE_INT, MPI_MINLOC, 0,
	     MPI_COMM_WORLD);

  if (myrank == 0) 
    {      
      printf("recv short array:\n");
      for (i=0; i<ntasks; i++) 
	printf("i:%d val:%d index:%d\n", i, srecvbuf[i].val, srecvbuf[i].rank);

      printf("recv long double array:\n");
      for (i=0; i<ntasks; i++) 
	printf("i:%d val:%Lf index:%d\n", i, ldrecvbuf[i].val, ldrecvbuf[i].rank);
    } 
  
  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;
    }
}
