#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* test for MPI_Reduce on MPI_PROD operation */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  /* test datatype unsigned */
  /* test on one-element sendbuf */
  unsigned int *us = (unsigned int*)malloc(sizeof(unsigned int));
  unsigned int *ur = (unsigned int*)malloc(sizeof(unsigned int));
  /* test on many-element sendbuf */
  unsigned int *usendbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*ntasks);
  unsigned int *urecvbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*ntasks);
  /* test datatype double */
  /* test on one-element sendbuf */
  double *ds = (double*)malloc(sizeof(double));
  double *dr = (double*)malloc(sizeof(double));
  /* test on many-element sendbuf */
  double *dsendbuf = (double*)malloc(sizeof(double)*ntasks);
  double *drecvbuf = (double*)malloc(sizeof(double)*ntasks);
  
  us[0] = myrank;
  init_buffer(usendbuf, ntasks, TUINT);
  ds[0] = (double)myrank;
  init_buffer(dsendbuf, ntasks, TDOUBLE);
  
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d us: ", myrank);
  printf("%d \n", us[0]);

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%f ", dsendbuf[i]);
  printf("\n");

  printf("myrank: %d us: ", myrank);
  printf("%f \n", ds[0]);


  MPI_Reduce(usendbuf, urecvbuf, ntasks, MPI_UNSIGNED, MPI_PROD, 0,
	     MPI_COMM_WORLD);
  MPI_Reduce(dsendbuf, drecvbuf, ntasks, MPI_DOUBLE, MPI_PROD, 0,
	     MPI_COMM_WORLD);
  MPI_Reduce(us, ur, 1, MPI_UNSIGNED, MPI_PROD, 0, MPI_COMM_WORLD);
  MPI_Reduce(ds, dr, 1, MPI_DOUBLE, MPI_PROD, 0, MPI_COMM_WORLD);

   
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%f ", dsendbuf[i]);
  printf("\n");

  if (myrank == 0) 
    {      
      printf("urecvbuf ");
      for (i=0; i<ntasks; i++) 
	printf("%d ", urecvbuf[i]);
      printf("\n");
      printf("ur ");
      printf("%d \n", ur[0]);
      printf("drecvbuf ");
      for (i=0; i<ntasks; i++) 
	printf("%f ", drecvbuf[i]);
      printf("\n");
      printf("dr ");
      printf("%f \n", dr[0]);
    }

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;
    }
}
