#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

/* general test for MPI_Reduce_scatter */

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)
     int argc;
     char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks, total_dcounts, total_ucounts;
  int *ucounts;
  int *dcounts;
  

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  
  /* initialize counts */
  ucounts = (int*)malloc(sizeof(int)*ntasks);
  total_ucounts = 0;
  for(i=0; i<ntasks; i++)
    {
      ucounts[i] = 1;
      total_ucounts += ucounts[i];
    }
  dcounts = (int*)malloc(sizeof(int)*ntasks);
  total_dcounts = 0;
  for(i=0; i<ntasks; i++)
    {
      dcounts[i] = ntasks*i;
      total_dcounts += dcounts[i];
      printf("dcounts=%d , i=%d\n", dcounts[i], i);
    }

  /* test datatype double */
  /* test on many-element sendbuf */
  double *dsendbuf = (double*)malloc(sizeof(double)*total_dcounts);
  double *drecvbuf = (double*)malloc(sizeof(double)*total_dcounts);
  /* test datatype unsigned */
  /* test on many-element sendbuf */
  unsigned int *usendbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*total_ucounts);
  unsigned int *urecvbuf = (unsigned int*)malloc(sizeof(unsigned
							int)*total_ucounts);
  
  init_buffer(usendbuf, total_ucounts, TUINT);
  init_buffer(dsendbuf, total_dcounts, TDOUBLE);

  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%f ", dsendbuf[i]);
  printf("\n");

  MPI_Reduce_scatter(usendbuf, urecvbuf, ucounts, MPI_UNSIGNED, MPI_PROD, 0,
	     MPI_COMM_WORLD);
  MPI_Reduce_scatter(dsendbuf, drecvbuf, dcounts, MPI_DOUBLE, MPI_PROD, 0,
	     MPI_COMM_WORLD);
   
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks*6; i++) 
    printf("%f ", dsendbuf[i]);
  printf("\n");    

  if(myrank ==0)
    {
      printf("myrank: %d urecvbuf: ", myrank);
      for (i=0; i<total_ucounts; i++) 
	printf("%d ", urecvbuf[i]);
      printf("\n");    
    }

  printf("urecvbuf ");
  for (i=0; i<ucounts[myrank]; i++) 
    printf("%d ", urecvbuf[i]);
  printf("\n");

  if(myrank ==0)
    {
      printf("myrank: %d drecvbuf: ", myrank);
      for (i=0; i<total_dcounts; i++) 
	printf("%f ", drecvbuf[i]);
      printf("\n");    
    }
  
  printf("myrank: %d drecvbuf: ", myrank);
  for (i=0; i<dcounts[myrank]; i++) 
    printf("%f ", drecvbuf[i]);
  printf("\n");
  
  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;
    }
}
