#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

enum types { TCHAR, TINT, TUINT, TDOUBLE };
void init_buffer(void *buf, int count, int type);

int main(argc, argv)

int argc;
char *argv[];

{	
  int myrank; /* SunOS has problems with this local decl. */
  MPI_Init(&argc, &argv);		/* initialize MPI */
  MPI_Comm_rank(MPI_COMM_WORLD,	&myrank);	/* process rank, 0 thru N-1 */

  int i;
  int ntasks, total_dcounts, total_ucounts;
  int *ucounts;
  int *dcounts;
  double *dsendbuf;
  double *drecvbuf;
  unsigned int *usendbuf, *urecvbuf;

  MPI_Comm_size(MPI_COMM_WORLD,   /* always use this */
		&ntasks);  
  
  /* initialize counts */
  ucounts = (int*)malloc(sizeof(int)*ntasks);
  total_ucounts = 0;
  for(i=0; i<ntasks; i++)
    {
      ucounts[i] = 1;
      total_ucounts += ucounts[i];
    }
  dcounts = (int*)malloc(sizeof(int)*ntasks);
  total_dcounts = 0;
  for(i=0; i<ntasks; i++)
    {
      dcounts[i] = ntasks*5;
      total_dcounts += dcounts[i];
    }

  /* test datatype double */
  /* test on many-element sendbuf */
  dsendbuf = (double*)malloc(sizeof(double)*dcounts[myrank]);
  drecvbuf = (double*)malloc(sizeof(double)*dcounts[myrank]);
  
  /* test datatype unsigned */
  /* test on one-element sendbuf */
  usendbuf = (unsigned int*)malloc(sizeof(unsigned int)*ucounts[myrank]);
  urecvbuf = (unsigned int*)malloc(sizeof(unsigned int)*ucounts[myrank]);
  
  init_buffer(usendbuf, ucounts[myrank], TUINT);
  init_buffer(dsendbuf, dcounts[myrank], TDOUBLE);

  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ucounts[myrank]; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<ntasks; i++) 
    printf("%f ", dsendbuf[i]);
  printf("\n");
  
  MPI_Scan(usendbuf, urecvbuf, ucounts[myrank], MPI_UNSIGNED, MPI_SUM, 0,
	     MPI_COMM_WORLD);
  MPI_Scan(dsendbuf, drecvbuf, dcounts[myrank], MPI_DOUBLE, MPI_SUM, 0,
     MPI_COMM_WORLD); 
   
  printf("myrank: %d usendbuf: ", myrank);
  for (i=0; i<ucounts[myrank]; i++) 
    printf("%d ", usendbuf[i]);
  printf("\n");

  printf("myrank: %d dsendbuf: ", myrank);
  for (i=0; i<dcounts[myrank]; i++) 
    printf("%f ", dsendbuf[i]);
    printf("\n");

  printf("myrank: %d urecvbuf: ", myrank);
  for (i=0; i<ucounts[myrank]; i++) 
    printf("%d ", urecvbuf[i]);
  printf("\n");    
  
  printf("myrank: %d drecvbuf: ", myrank);
  for (i=0; i<dcounts[myrank]; i++) 
    printf("%f ", drecvbuf[i]);
  printf("\n");    

  MPI_Finalize();			/* cleanup MPI */
  exit(0);
}

/* function for initializing buffers with data depending on type */
void init_buffer(void *buf, int count, int type)
{
  long i;
  switch (type)
    {
    case TCHAR:
      for(i=0; i<count; i++)
        ((char*)buf)[i]=(char)(i%26)+65;
      break;

    case TINT:
      for(i=0; i<count; i++)
        ((int*)buf)[i]=i+1;
      break;

    case TUINT:
      for(i=0; i<count; i++)
        ((unsigned int*)buf)[i]=(unsigned int)i+1;
      break;

    case TDOUBLE:
      for(i=0; i<count; i++)
        ((double*)buf)[i]=(double)i+1;
      break;
    }
}
