/*
 * genprocgroup <name> <instances>:
 *
 * Generate a procgroup file given a name, a number of instances that
 * should be started. The file, named <name>.procgroup will have the
 * following form:
 *
 *   local 0
 *   localhost 1 ${PWD}/<name>		; instance 1
 *   localhost 1 ${PWD}/<name>		; instance 2
 *   .
 *   .
 *   .
 *   localhost 1 ${PWD}/<name>		; instance n
 */
#include <unistd.h>
#include <stdio.h>
#include <limits.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

static const char *firstLine = "local 0\n";
static const char *filenameTemplate = "%s.procgroup";

int main (int argc, char *argv[]) {
  char *name = argv[1];
  char cwd[PATH_MAX];
  char filename[PATH_MAX];

  if (argc != 3) {
    fprintf (stderr, "Usage: %s <name> <instances>\n", argv[0]);
    return EINVAL;
  }

  snprintf (filename, sizeof(filename), filenameTemplate, name);
  FILE *fp = fopen(filename, "w+");
  if (NULL == fp) {
    perror ("fopen");
    abort ();
  }

  int iretval = fwrite (firstLine, 1, strlen(firstLine), fp);
  if (iretval != strlen(firstLine)) {
    perror ("fwrite");
    abort ();
  }

  char *sretval = getcwd (cwd, sizeof(cwd));
  if (NULL == sretval) {
    perror ("getcwd");
    abort ();
  }

  int i;
  for (i = atoi(argv[2]); i > 0; i--) {
    fprintf (fp, "localhost 1 %s/%s\n", cwd, name);
  }

  fclose (fp);
  return 0;
}

