#include <stdio.h>
#include <assert.h>
#include "mpi.h"

/*
 * Name:	hello_mpi1a
 *
 * Purpose:	Verify one node calling MPI_Finalize() before another works in
 *		the presence of buffered sends.
 *
 * Slaves:	> 1, so that some MPI internal setup communication must have
 *		occurred.
 */
int main( int argc, char *argv[] ) {
  int		myrank, maxrank, work;
  MPI_Status	status;

  MPI_Init( &argc, &argv );		/* initialize MPI */
  MPI_Comm_rank( MPI_COMM_WORLD,	/* always use this */
      &myrank );			/* process rank, 0 thru N-1 */
  MPI_Comm_size(MPI_COMM_WORLD,		/* always use this */
      &maxrank );			/* #processes in application = N */
  
  if ( 0 == myrank ) {
    int i;
    work = 0xf1ee;
    for ( i = 1; i < maxrank; i++ ) {
      MPI_Send( &work, 1, MPI_INT, i, 5, MPI_COMM_WORLD );
    }
  } else {
    /*
     * Sleep for twice the number of seconds of this processes rank. This
     * gives rank 0 a chance to enter MPI_Finalize before this rank even
     * receives its first message.
     */
    sleep( 2 * myrank );

    work = 0;
    MPI_Recv( &work, 1, MPI_INT, 0, 5, MPI_COMM_WORLD, &status );

    assert( 0 == status.MPI_SOURCE );
    assert( 5 == status.MPI_TAG );
    int cnt;
    MPI_Get_count( &status, MPI_INT, &cnt );
    assert( 1 == cnt );
    assert( 0xf1ee == work );
  }

  MPI_Finalize();			/* cleanup MPI */
  return 0;
}
