#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "mpi.h"

#define	MAX_BUF_SIZE	100

/*
 * Name:	hello_mpi1b
 *
 * Purpose:	Verify that receive operations only read as much data as
 *		requested, and make sure short messages don't result in
 *		larger buffers being changed beyond the end of the provided
 *		data, and that provided buffers shorter than the received
 *		don't cause problems with buffer overflows or data reception.
 *
 * Slaves:	= 1, to verify send and receive
 */
int main( int argc, char *argv[] ) {
  int		myrank, *work, i;
  int		work_size = MAX_BUF_SIZE * sizeof(int);
  MPI_Status	status;

  MPI_Init( &argc, &argv );		/* initialize MPI */
  MPI_Comm_rank( MPI_COMM_WORLD,	/* always use this */
      &myrank );			/* process rank, 0 thru N-1 */

  work = malloc( work_size );
  assert( work );

  if ( 1 == myrank ) {
    for ( i = 0; i <= MAX_BUF_SIZE; i++ ) {
      memset( work, i, work_size );
      MPI_Send( work, i, MPI_INT, 0, 5, MPI_COMM_WORLD );
    }
    for ( i = 0; i <= MAX_BUF_SIZE; i++ ) {
      memset( work, i, work_size );
      MPI_Send( work, i, MPI_INT, 0, 5, MPI_COMM_WORLD );
    }
  } else {
    assert( 0 == myrank );
    int rsize;
    for ( i = 0; i <= MAX_BUF_SIZE; i++ ) {
      memset( work, -1, work_size );
      MPI_Recv( work, MAX_BUF_SIZE, MPI_INT, 1, 5, MPI_COMM_WORLD, &status );

      assert( 1 == status.MPI_SOURCE );
      assert( 5 == status.MPI_TAG );
      int cnt;
      MPI_Get_count( &status, MPI_INT, &cnt );
      assert( i == cnt );
      int pattern;
      memset( &pattern, i, sizeof(pattern) );
      int j;
      for ( j = 0; j < cnt; j++ ) {
#ifdef DEBUG
	printf( "work[%d] = %#lx", j, work[j] );
	if ( work[j] != pattern )
	  printf( " expected: %#lx\n", pattern );
	else
	  printf( "\n" );
#else
	assert( work[j] == pattern );
#endif
      }
      for ( j = cnt; j < MAX_BUF_SIZE; j++ ) {
#ifdef DEBUG
	printf( "work[%d] = %#lx", j, work[j] );
	if ( work[j] != -1 )
	  printf( " expected: %#lx\n", -1 );
	else
	  printf( "\n" );
#else
	assert( -1 == work[j] );
#endif
      }
#ifdef DEBUG
      printf( "\n" );
#endif
    }
    for ( i = 0, rsize = MAX_BUF_SIZE; i <= MAX_BUF_SIZE; i++, rsize-- ) {
      memset( work, -1, work_size );
      MPI_Recv( work, rsize, MPI_INT, 1, 5, MPI_COMM_WORLD, &status );

      assert( 1 == status.MPI_SOURCE );
      assert( 5 == status.MPI_TAG );
      int cnt;
      MPI_Get_count( &status, MPI_INT, &cnt );
      assert( i == cnt );
      int pattern;
      memset( &pattern, i, sizeof(pattern) );
      int j;
      int valid = cnt > rsize ? rsize : cnt;
      for ( j = 0; j < valid; j++ ) {
#ifdef DEBUG
	printf( "work[%d] = %#lx", j, work[j] );
	if ( work[j] != pattern )
	  printf( " expected: %#lx\n", pattern );
	else
	  printf( "\n" );
#else
	assert( work[j] == pattern );
#endif
      }
      for ( j = valid; j < MAX_BUF_SIZE; j++ ) {
#ifdef DEBUG
	printf( "work[%d] = %#lx", j, work[j] );
	if ( work[j] != -1 )
	  printf( " expected: %#lx\n", -1 );
	else
	  printf( "\n" );
#else
	assert( -1 == work[j] );
#endif
      }
#ifdef DEBUG
      printf( "\n" );
#endif
    }
  }

  MPI_Finalize();			/* cleanup MPI */
  return 0;
}
