#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "mpi.h"

#define CASES	6

static void doIprobe( int rank, int tag ) {
  MPI_Status status;
  int count, flag;

  for ( flag = 0; !flag; ) {
    MPI_Iprobe( rank, tag, MPI_COMM_WORLD, &flag, &status );
  }
  assert( 1 == flag );
  assert( status.MPI_SOURCE == rank );
  assert( status.MPI_TAG == tag );
  MPI_Get_count( &status, MPI_INT, &count );
  assert( 1 == count );
}

static void doProbe( int rank, int tag ) {
  MPI_Status status;
  int count;

  MPI_Probe( rank, tag, MPI_COMM_WORLD, &status );
  assert( status.MPI_SOURCE == rank );
  assert( status.MPI_TAG == tag );
  MPI_Get_count( &status, MPI_INT, &count );
  assert( 1 == count );
}

static void testIprobeRank1( void ) {
  int thisCase;

  for ( thisCase = 1; thisCase <= CASES; thisCase++ ) {
    int tag = (thisCase * 1000) + 1;
    MPI_Send( &tag, 1, MPI_INT, 0, tag, MPI_COMM_WORLD );
    tag++;
    MPI_Send( &tag, 1, MPI_INT, 0, tag, MPI_COMM_WORLD );
  }
}

static void testIprobeRank0( void ) {
  int thisCase;
  for ( thisCase = 1; thisCase <= CASES; thisCase++ ) {
    printf( "Case %d\n", thisCase );
    int tag1 = (thisCase * 1000) + 1;
    int tag2 = tag1 + 1;
    switch ( thisCase ) {
      case 1: {
	doProbe( 1, tag1 );
	doProbe( 1, tag2 );
	doIprobe( 1, tag2 );
	break;
      }
      case 2: {
	doIprobe( 1, tag1 );
	doIprobe( 1, tag2 );
	break;
      }
      case 3: {
	doIprobe( 1, tag1 );
	doProbe( 1, tag2 );
	break;
      }
      case 4: {
	doProbe( 1, tag2 );
	doIprobe( 1, tag2 );
	doProbe( 1, tag1 );
	break;
      }
      case 5: {
	doIprobe( 1, tag2 );
	doIprobe( 1, tag1 );
	break;
      }
      case 6: {
	doIprobe( 1, tag2 );
	doProbe( 1, tag1 );
	break;
      }
      default: {
	fprintf( stderr, "%s: invalid case encountered %d\n",
	    __func__, thisCase );
	abort();
      }
    }
  }
}

/*
 * Name:	hello_mpi7
 *
 * Purpose:	show that Iprobe is "best effort", meaning a busy loop using
 *		Iprobe() behaves the same as a Probe().
 *
 *		    Given: Send T1
 *			   Send T2
 *
 *		    The following sets should work:
 *
 *		    1. P  T1  2. Ip T1  3. Ip T1
 *		       P  T2     Ip T2     P  T2
 *		       Ip T2
 *		       -----     -----     -----
 *		    4. P  T2  5. Ip T2  6. Ip T2
 *		       Ip T2     Ip T1     P  T1
 *		       P  T1
 *
 * Slaves:	1
 */
int main( int argc, char *argv[] ) {
  int myrank;

  MPI_Init( &argc, &argv );	/* initialize MPI */
  // always use this process rank, 0 thru N-1
  MPI_Comm_rank( MPI_COMM_WORLD, &myrank );

  if ( 1 == myrank ) {
    testIprobeRank1();
  }
  else {
    assert( 0 == myrank );
    testIprobeRank0();
  }

  MPI_Finalize();			/* cleanup MPI */
  return 0;
}
