#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <pthread.h>
#include "mpi.h"

static void *check( void *arg ) {
  int flag;
  assert( MPI_SUCCESS == MPI_Is_thread_main( &flag ) );
  assert( !flag );
  return NULL;
}

/*
 * Name:	th_init1
 *
 * Purpose:	Verify behavior of MPI_Init_thread(), MPI_Query_thread() and
 *		MPI_Is_thread_main().
 *
 * Slaves:	Doesn't matter.
 */
int main( int argc, char *argv[] ) {
  int provided, provided2, flag;
  pthread_t thread;

  /* Verify the definitions of these symbols are monotonically increasing */
  assert( MPI_THREAD_SINGLE < MPI_THREAD_FUNNELED );
  assert( MPI_THREAD_FUNNELED < MPI_THREAD_SERIALIZED );
  assert( MPI_THREAD_SERIALIZED < MPI_THREAD_MULTIPLE );

  /* initialize MPI using the thread initialization version */
  assert( MPI_SUCCESS == MPI_Init_thread(
	      &argc, &argv, MPI_THREAD_MULTIPLE, &provided ) );
  assert( MPI_SUCCESS == MPI_Query_thread( &provided2 ) );
  assert( provided == provided2 );
  assert( MPI_SUCCESS == MPI_Is_thread_main( &flag ) );
  assert( flag );

  assert( 0 == pthread_create( &thread, NULL, check, NULL ) );
  assert( 0 == pthread_join( thread, NULL ) );

  MPI_Finalize();			/* cleanup MPI */
  return 0;
}
