 /**********************************************************************
  * MPINU				                               *
  * Copyright (c) 2004-2005 Gene Cooperman <gene@ccs.neu.edu>          *
  *                                                                    *
  * This library is free software; you can redistribute it and/or      *
  * modify it under the terms of the GNU Lesser General Public         *
  * License as published by the Free Software Foundation; either       *
  * version 2.1 of the License, or (at your option) any later version. *
  *                                                                    *
  * This library is distributed in the hope that it will be useful,    *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
  * Lesser General Public License for more details.                    *
  *                                                                    *
  * You should have received a copy of the GNU Lesser General Public   *
  * License along with this library (see file COPYING); if not, write  *
  * to the Free Software Foundation, Inc., 59 Temple Place, Suite      *
  * 330, Boston, MA 02111-1307 USA, or contact Gene Cooperman          *
  * <gene@ccs.neu.edu>.                                                *
  **********************************************************************/

#include "mpi.h"
#include "mpiimpl.h"

#define true 1
#define false 0
#define bool int
#define UINT32 int
// #include "../../../src/tivlog.h"
#include "tivlog.h"

/*====================================================================*/
/* MPI Utility Functions */

int MPI_Comm_rank( MPI_Comm comm, int *rank )
{ *rank = MPINU_myrank;
  return 0;
}

int MPI_Comm_size( MPI_Comm comm, int *size )
{ *size = 1 + MPINU_num_slaves;
  return 0;
}

int MPI_Get_processor_name( char *name, int *resultlen )
{ gethostname( name, MPINU_MAX_PROCESSOR_NAME );
  *resultlen = strlen( name );
  if ( *resultlen < MPINU_MAX_PROCESSOR_NAME ) return 0;
  else return -1;
}

int MPI_Abort( MPI_Comm comm, int errorcode )
{ printf("MPI_Abort called with errorcode: %d\n", errorcode);
  abort();
  return( 0 ); /* return for lint */
}

/* MPI_TAG_UB, MPI_HOST <- MPI_PROC_NULL, MPI_IO (myrank), MPI_WTIME_IS_GLOBAL
								<- 0
  attr_value can be (int *) */
int MPI_Attr_get (
     MPI_Comm comm,
     int keyval,
     void *attr_value,
     int *flag)
{ *flag = 1;
  switch( keyval ) {
    case MPI_TAG_UB: *(int *)attr_value = ~(1u << (8 * sizeof(INT) - 1) );
      break;
    case MPI_HOST: *(int *)attr_value = MPI_PROC_NULL;
      break;
    case MPI_IO: *(int *)attr_value = MPINU_myrank;
      break;
    case MPI_WTIME_IS_GLOBAL: *(int *)attr_value = 0;
      break;
    default: *flag = 0;
  }
  return( 0 );
}

int MPI_Error_string( int errorcode, char *string, int *resultlen )
{ static char err[] = "MPINU:  MPI_Error_string not implemented.\n";
  strncpy(string, err, MPI_MAX_ERROR_STRING);
  *resultlen = strlen(err);
  return( 0 );
}

/*====================================================================*/
/* Utilities for implementation */

int MPINU_rank_from_socket(int s) {
  int i;
  for (i = 0; i <= MPINU_num_slaves; i++)
    if (MPINU_rank_socket_get(i) == s)
      return i;
  return -1; /* socket not found */
}

int MPINU_rank_socket_get(int rank) {
  return MPINU_pg_array[rank].sd;
}

void MPINU_rank_socket_init(int rank) {
  MPINU_pg_array[rank].sd = PG_NOSOCKET;
}

void MPINU_rank_socket_set(int rank, int sd) {
  int *r_sd = &MPINU_pg_array[rank].sd;
  if ( *r_sd == sd )
    return;
  /*
   * Acquire the global lock to synchronize access to MPINU_max_sd and
   * MPINU_fdset. It is assumed the caller holds the proper lock for
   * changing the rank's socket descriptor.
   */
  assert( 0 == pthread_mutex_lock( &MPINU_global ) );
  if ( PG_NOSOCKET == sd ) {
    assert( *r_sd <= MPINU_max_sd );
    FD_CLR( *r_sd, &MPINU_fdset );
  } else {
    if ( *r_sd != PG_NOSOCKET ) {
      FD_CLR( *r_sd, &MPINU_fdset );
      CALL_CHK( close, ( *r_sd ) );
    }
    FD_SET( sd, &MPINU_fdset );
    if ( sd > MPINU_max_sd )
      MPINU_max_sd = sd;
  }
  *r_sd = sd;
  assert( 0 == pthread_mutex_unlock( &MPINU_global ) );
}
